# Copyright 2005-2007, Google, Inc.

# This software is provided as an example of using the Ruby interface
# to SketchUp.

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------

# This adds some useful functions to the Utilities menu in SketchUp
require 'sketchup.rb'



def simplyContours

    index = Sketchup.version.index(".")
    majorVersion =  Sketchup.version.slice(0..index)
    majorVersion = majorVersion.to_i

    if majorVersion<5
    
        UI.messagebox("This script will only run on SketchUp 5 or greater.")
        return
    
    end

    model = Sketchup.active_model
    
    #define everything as one undo operation
    model.start_operation "Simplify Contours"

    result = UI.inputbox ["Angle (deg)"], [10], "Simplify Angle"
    angle = (180-result[0])/57.296
    
    ss = Sketchup.active_model.selection

    #Get rid of any selected entities that are not edges
    remove = []

    ss.each do |entity|
    
        if !entity.kind_of?(Sketchup::Edge)
        
            remove.push(entity)
        
        end

    end

    ss.remove(remove)
    
    edges = []
    
    ss.each do |edge|
    
        edges.push(edge)
        
    end
    
    puts "Number Edges = " + ss.length.to_s
    
    parent = edges[0].parent.entities
    usedSet = Set.new
    contours = []
    
    edges.each do |edge|
    
        if !usedSet.include?(edge)
    
            orderedPoints = []
        
            curve = edge.curve
            
            if curve
            
                curve.vertices.each do |vert|
                
                    orderedPoints.push(vert.position)
                    
                end
                
                curve.edges.each do |edge|
                
                    usedSet.insert(edge)
                    
                end
                
                contours.push(orderedPoints)
        
            else
            
                #puts "using manual ordering"
                
                connectedEdges = edge.all_connected

                #remove all the non-edges
                for i in (0..connectedEdges.length-1)
                    if !connectedEdges[i].kind_of?(Sketchup::Edge)
                        connectedEdges[i] = nil
                    end
                end
                connectedEdges.compact!
                            
                bNoTVerts = true
                connectedEdges.each do |edge|
                
                    if edge.start.edges.length>2
                        bNoTVerts = false
                        break
                    end
                    
                    if edge.end.edges.length>2
                        bNoTVerts = false
                        break
                    end
                
                end
                
                if bNoTVerts
                
                    orderedPoints = OrderConnectedEdges(connectedEdges)
                
                end
                
                #mark all these edges as used
                connectedEdges.each do |edge|
                
                    usedSet.insert(edge)
                
                end
                
                if orderedPoints.length>1
                
                    contours.push(orderedPoints)
                    
                end
            
            end
            
        end
    
    end
        
    usedEdges = usedSet.to_a
    parent.erase_entities(usedEdges)
    
    count = 0
    contours.each do |contour|
    
        simpleContour = SimplifyCurve(contour, angle)
        parent.add_curve(simpleContour)
        count+=1
        tellPctComplete(count,contours.length) 
    
    end
    
    model.commit_operation

end

def SimplifyCurve(orderedPoints, angle)

    simplePoints = []

    simplePoints.push(orderedPoints[0])
    lastPoint = orderedPoints[0]
    
    for i in (1..orderedPoints.length-2)
    
        currentPoint = orderedPoints[i]
        nextPoint = orderedPoints[((1+i)%orderedPoints.length)]
    
        vec1 = lastPoint - currentPoint
        vec2 = nextPoint - currentPoint
    
        if (vec1.angle_between(vec2)>angle)
        
            #skip this vertex entirely
        
        else
        
            simplePoints.push(currentPoint)
            lastPoint = currentPoint 
        
        end
        
    end
    
    simplePoints.push(orderedPoints.last)

    return simplePoints
  
end

def OrderConnectedEdges(edges)

    #Scan through looking for an edge vertex that is only
    #connected to a single edge.  If non are found, then 
    #this is a closed loop
    orderedPoints = []
    
    if edges.length==1
        orderedPoints.push(edges[0].start.position)
        orderedPoints.push(edges[0].end.position)
        return orderedPoints
    end
    
    startVert = nil
    edges.each do |edge|
    
        if edge.start.edges.length==1
            startVert = edge.start
            break
        end

        if edge.end.edges.length==1
            startVert = edge.end
            break
        end

    end
    
    #closed Loop case
    if startVert==nil
        startVert = edges[0].start
    end

    orderedPoints.push(startVert.position)
    lastEdge = startVert.edges[0]
    currentVert = lastEdge.other_vertex(startVert)
    
    while (true)

        orderedPoints.push(currentVert.position)
        
        #find the next edge
        currentVert.edges.each do |edge|
        
            if edge!=lastEdge
                lastEdge = edge
                break
            end
            
        end
        
        #find the next vertex
        currentVert = lastEdge.other_vertex(currentVert)
        
        #push this on if this is our last point
        if currentVert.edges.length==1
            orderedPoints.push(currentVert.position)
            break
        end
        
        if currentVert==startVert
            orderedPoints.push(currentVert.position)
            break
        end
                
    end
    
    return orderedPoints

end

def tellPctComplete(iteration,total) 
    linechar = "-"                          # Set default line building character.
    progresschar = ">"                      # Set default moving indicator.
    pct = (iteration*100)/total             # Calculate percentage complete.
    pct = 1 if pct <= 0                     # round up to 1% if anything less than 1%.
    initial_block = linechar * 100          # Default progress bar line sequence.
    current_block = "|" << initial_block[0,pct-1] << progresschar << initial_block[pct,initial_block.length] << "|"
    Sketchup.set_status_text(current_block << "   " << (pct.to_s)<<"%.")
    return
end    # tellPctComplete







# This funtion will create a face from the edges in the selection set.
# There must be at least three Edges selected.
# It will show an error message if a face couldn't be created from the edges.
def create_face_from_selection
    ss = Sketchup.active_model.selection
    
    # Get an Array of all of the selected Edges
    edges = ss.find_all { |e| e.kind_of?(Sketchup::Edge) }
    
    # We need at least 3 Edges
    if( edges.length < 3 )
        UI.messagebox($uStrings.GetString("You must select at least three Edges"))
        return nil
    end
    
    # Try to create a Face from the Edges in the active component
    face = nil
    begin
        face = Sketchup.active_model.active_entities.add_face edges
    rescue
    end
    
    # If a Face wasn't created, then there should be an error message
    # telling what the problem was
    if( not face )
        if( $! )
            msg = $!.message
        else
            msg = $uStrings.GetString("Create Face from Edges failed")
        end
        UI.messagebox msg
    end
    
    # Return the Face that was created
    face
end

#-------------------------------------------------------------------------
# This example show how you can write a simple tool in Ruby.  To implement
# a tool, you have to implement the methods that you want to handle.
# In this example the following methods are implementd:
# onMouseMove is called when a mouse move event is received.
# draw is handled to draw the tool

# This tool show coordinates of points and screen positions.
# It also shows lengths of selected Edges or areas of selected Faces.

# To see this tool in action, type the following in the Ruby console window:
# Sketchup.active_model.select_tool TrackMouseTool.new

class TrackMouseTool

# The activate method is called when a tool is first activated.  It is not
# required, but it is a good place to initialize stuff.
def activate
    @ip = Sketchup::InputPoint.new
    @iptemp = Sketchup::InputPoint.new
    @displayed = false
end

# onMouseMove is called whenever SketchUp gets a mouse move event.  It is called
# a lot, so you should try to make it as efficient as possible.
def onMouseMove(flags, x, y, view)
    # show the screen position in the VCB
    Sketchup::set_status_text("#{x}, #{y}", SB_VCB_VALUE)
    
    # get a position in the model and show it in a tooltip
    @iptemp.pick view, x, y
    if( @iptemp.valid? )
        changed = @iptemp != @ip
        @ip.copy! @iptemp
        pos = @ip.position;
        
        # get the text for the position
        msg = @ip.tooltip
        if( msg.length > 0 )
            msg << " "
        end
        #msg << pos.to_s
        msg << "( #{Sketchup.format_length(pos.x)},#{Sketchup.format_length(pos.y)},#{Sketchup.format_length(pos.z)} )"

        
        # See if it is on any special geometry
        if( @ip.vertex == nil )
            if( @ip.edge )
                if( @ip.depth > 0 )
                    length = @ip.edge.length(@ip.transformation)
                else
                    length = @ip.edge.length
                end
                msg <<  "\n       " + $uStrings.GetString("length") + " = #{Sketchup.format_length(length)}"
            elsif( @ip.face )
                if( @ip.depth > 0 )
                    area = @ip.face.area(@ip.transformation)
                else
                    area = @ip.face.area
                end
                msg << "\n       " + $uStrings.GetString("area") + " = #{Sketchup.format_area(area)}"
            end
        end

        msg2 = msg.gsub(/\n/,' ')
        Sketchup::set_status_text msg2
        
        # set the tooltip to show this message
        view.tooltip = msg
        
        # see if we need to update the display for this point
        if( changed and (@ip.display? or @displayed) )
            # This tells the view that we want it to update itself
            view.invalidate
        end
    end
    
end

# onLButtonDown is called when the user presses the left mouse button
def onLButtonDown(flags, x, y, view)
    Sketchup::set_status_text $uStrings.GetString("Left button down at") + " (#{x}, #{y})"
end

# onLButtonUp is called when the user releases the left mouse button
def onLButtonUp(flags, x, y, view)
    Sketchup::set_status_text $uStrings.GetString("Left button up at") + " (#{x}, #{y})"
end

# draw is optional.  It is called on the active tool whenever SketchUp
# needs to update the screen.
# in this case, we display an input point if needed
def draw(view)
    if( @ip.valid? && @ip.display? )
        @ip.draw view
        @displayed = true
    else
        @displayed = false
    end
end

def getInstructorContentDirectory
    "Query"
end

end

#-----------------------------------------------------------------------------
# Make selected components unique.
# This looks at all selected components.
def make_selected_components_unique

    # First get all selected components
    ss = Sketchup.active_model.selection
    components = ss.find_all {|e| e.kind_of?(Sketchup::ComponentInstance) }
    
    # Create a Hash.  The keys are the guids of all of the component definitions
    # for selected components.  The values are arrays of all of the component instances
    # that have that definition
    definitions = {}
    for component in components
        key = component.definition.guid
        a = definitions[key]
        if( a )
            a.push component
        else
            definitions[key] = [component]
        end
    end
    
    # Now make each collection of component instances have a new unique definition
    definitions.each_value do |a|
        # a is an array of component instances that all share the same definition
        # We will make the first one have a new unique definition, and all of the
        # other ones use that new definition
        definition = nil
        for component in a
            if( definition )
                # We already have a new definition.  Make this component use it
                component.definition = definition
            else
                # Make the component have a unique definition and save it
                component.make_unique
                definition = component.definition
            end
        end
    end

    true
end








# Create a grids

require 'sketchup.rb'
require 'parametric.rb'

#=============================================================================

class Grid < Parametric

def create_entities(data, container)

    dx = data["dx"].to_l
    dy = data["dy"].to_l
    nx = data["nx"].to_i
    ny = data["ny"].to_i
    
    # create the vertical lines
    pt = Geom::Point3d.new ORIGIN
    pt2 = pt + [0, dy*ny, 0]
    offset = [dx, 0, 0]
    for i in 0..nx do
        container.add_cline pt, pt2, "..."
        pt += offset
        pt2 += offset
    end
    
    # create the horizontal lines
    pt.set!(ORIGIN)
    pt2 = pt + [dx*nx, 0, 0]
    offset = [0, dy, 0]
    for i in 0..ny do
        container.add_cline pt, pt2, "..."
        pt += offset
        pt2 += offset
    end
    
end

def default_parameters
    defaults = {"dx", 8.feet, "dy", 8.feet, "nx", 4, "ny", 4}
    defaults
end

end

#=============================================================================

class GridTool

@@dx = 1.feet
@@dy = 1.feet

def initialize
    @ip = Sketchup::InputPoint.new
    @ip1 = Sketchup::InputPoint.new
    reset
end

def reset
    @pts = []
    @state = 0
    @ip1.clear
    @drawn = false
    Sketchup::set_status_text "Click for start point"
    @shift_down_time = Time.now
    @nx = 0
    @ny = 0
end

def activate
    self.reset
    @drawn = false
    Sketchup::set_status_text "Spacing", SB_VCB_LABEL
    Sketchup::set_status_text "#{@@dx}", SB_VCB_VALUE
end

def deactivate(view)
    view.invalidate if @drawn
end

def set_current_point(x, y, view)
    if( !@ip.pick(view, x, y, @ip1) )
        return false
    end
    need_draw = true
    
    # Set the tooltip that will be displayed
    view.tooltip = @ip.tooltip
        
    # Compute points
    case @state
    when 0
        @pts[0] = @ip.position
        @pts[4] = @pts[0]
        need_draw = @ip.display? || @drawn
    when 1
        @pts[1] = @ip.position
    when 2
        pt1 = @ip.position
        pt2 = pt1.project_to_line @pts
        vec = pt1 - pt2
        @pts[2] = @pts[1] + vec
        @pts[3] = @pts[0] + vec
    end

    view.invalidate if need_draw
end

def onMouseMove(flags, x, y, view)
    self.set_current_point(x, y, view)
end

def create_grid
    # check for zero size
    if( @nx > 0 && @ny > 0 )
        data = {}
        data["nx"] = @nx
        data["ny"] = @ny
        data["dx"] = @@dx
        data["dy"] = @@dy
        
        # compute the transformation to move it to the right location
        origin = @pts[0]
        xaxis = @pts[1] - @pts[0]
        yaxis = @pts[3] - @pts[0]
        zaxis = xaxis.cross(yaxis)
        t = Geom::Transformation.axes(origin, xaxis, yaxis, zaxis)
        
        # Create the grid
        grid = Grid.new data, t
        
    end
end

def increment_state
    @state += 1
    case @state
    when 1
        @ip1.copy! @ip
        Sketchup::set_status_text "Click for second point"
    when 2
        @ip1.clear
        @pts[2] = @pts[1]
        @pts[3] = @pts[0]
        Sketchup::set_status_text "Click for third point"
    when 3
        self.create_grid
    end
end

def onLButtonDown(flags, x, y, view)
    self.set_current_point(x, y, view)
    self.increment_state
    view.lock_inference
end

def onLButtonUp(flags, x, y, view)
    Sketchup.active_model.select_tool nil if @state == 3
end

def onCancel(flag, view)
    view.invalidate if @drawn
    self.reset
end

# This is called when the user types a value into the VCB
def onUserText(text, view)
    #TODO: Add parsing for different values fo dx and dy
    # The user can enter values for dx and dy
    # The user may type in something that we can't parse as a length
    # so we set up some exception handling to trap that
    begin
        value = text.to_l
    rescue
        # Error parsing the text
        UI.beep
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value
    
    if( value > 0.0 )
        @@dx = value
        @@dy = value
        view.invalidate
    end
end

def getExtents
    bb = Geom::BoundingBox.new
    case @state
    when 0
        # We are getting the first point
        if( @ip.valid? && @ip.display? )
            bb.add @ip.position
        end
    when 1
        bb.add @pts[0]
        bb.add @pts[1]
    when 2
        bb.add @pts
    end
    bb
end

def draw_grid(view)

    # round the length to an even number of grid squares
    xvec = @pts[1] - @pts[0]
    width = xvec.length
    @nx = (width / @@dx).to_i
    width = @nx * @@dx
    xvec.length = width if( width > 0 )

    if( @state > 1 )
        yvec = @pts[3] - @pts[0]
        height = yvec.length
        @ny = (height / @@dy).to_i
        height = @ny * @@dy
        yvec.length = height if( height > 0 )
    else
        @ny = 0
    end

    # Check for special cases where there is not really anything to draw
    if( @ny < 1 )
        return if @nx < 1
        
        # Draw a single line to avoid confusion in the case where
        # we are just starting to drag out the grid
        view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
        
        # Draw tick marks
        offset = xvec.clone
        viewdir = view.camera.direction
        vec = offset.cross viewdir
        return if vec.length == 0.0
        vec.length = view.pixels_to_model 6, @pts[0]
        offset.length = @@dx
        pt1 = @pts[0]
        pt2 = pt1 + vec
        view.line_stipple = 0
        for i in 0..@nx do
            view.draw(GL_LINE_STRIP, pt1, pt2)
            pt1 += offset
            pt2 += offset
        end
        
        return
        
    elsif( @nx < 1 )
        view.draw(GL_LINE_STRIP, @pts[0], @pts[3])
        return
    end
    
    view.drawing_color = "black"
    
    # draw the vertical lines
    pt1 = @pts[0]
    pt2 = pt1 + yvec
    offset = xvec.clone
    offset.length = @@dx
    for i in 0..@nx do
        view.draw(GL_LINE_STRIP, pt1, pt2)
        pt1 += offset
        pt2 += offset
    end
    
    # draw the horizontal lines
    pt1 = @pts[0]
    pt2 = pt1 + xvec
    offset = yvec
    offset.length = @@dy
    for i in 0..@ny do
        view.draw(GL_LINE_STRIP, pt1, pt2)
        pt1 += offset
        pt2 += offset
    end
    
    true
end

def draw(view)
    @drawn = false
    
    # Show the current input point
    if( @ip.valid? && @ip.display? )
        @ip.draw(view)
        @drawn = true
    end

    # show the rectangle
    view.line_stipple = "..."
    if( @state == 1 )
        # just draw a line from the start to the end point
        view.set_color_from_line(@ip1, @ip)
        #inference_locked = view.inference_locked?
        #view.line_width = 3 if inference_locked
        @drawn = draw_grid(view)
        #view.line_width = 1 if inference_locked
    elsif( @state > 1 )
        @drawn = draw_grid(view)
    end
    view.line_stipple = 0
end

def onKeyDown(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            view.lock_inference
        elsif( @state == 1 )
            view.lock_inference @ip, @ip1
        else
            view.lock_inference @ip
        end
    end
end

def onKeyUp(key, rpt, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end

end # of class GridTool

# =========================================


def edge_selected
# Check if face is in selection set - for context menu
  lookup = false
  selection_set = Sketchup.active_model.selection
  selection_set.each {|entity|
    if entity.typename == "Edge"
      lookup = true
    
    end
  }
  
    selection_set.each {|entity|
    if entity.typename == "Face"
      lookup = false
    
    end
  }
  
  return lookup
end


# =========================================


#----------------------------------------------------------------------------
# Add things to the Utilities menu
if( not $utilities_menu_loaded )
    #add_separator_to_menu("Tools")
    #UI.menu("Tools").add_item($uStrings.GetString("Utilities"))

    UI.menu("Tools").add_item($uStrings.GetString("Bound Contours")) { simplyContours }
    #UI.menu("Tools").add_item($uStrings.GetString("Create Face")) { create_face_from_selection }
    #UI.menu("Tools").add_item($uStrings.GetString("Query Tool")) { Sketchup.active_model.select_tool TrackMouseTool.new }
    UI.menu("Tools").add_item($uStrings.GetString("Grid")) { Sketchup.active_model.select_tool GridTool.new }

   # Add to the context menu
  UI.add_context_menu_handler do |menu|
    if( edge_selected )
      menu.add_item("Create face") { create_face_from_selection }
    end
  end

    $utilities_menu_loaded = true
end
