
require 'sketchup.rb'
require 'parametric.rb'

#=============================================================================

# Function for generating points on a circle
def points_on_circle(center, normal, radius, numseg)
    # Get the x and y axes
    axes = Geom::Vector3d.new(normal).axes
    center = Geom::Point3d.new(center)
    xaxis = axes[0]
    yaxis = axes[1]
    
    xaxis.length = radius
    yaxis.length = radius

    # compute the points
    da = (Math::PI * 2) / numseg
    pts = []
    for i in 0...numseg do
        angle = i * da
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        vec = Geom::Vector3d.linear_combination(cosa, xaxis, sina, yaxis)
        pts.push(center + vec)
    end
    
    # close the circle
    pts.push(pts[0].clone)

    pts
end

#=============================================================================

class Box < Parametric

def create_entities(data, container)

    w = data["w"].to_l
    d = data["d"].to_l
    h = data["h"].to_l
    
    pts = [[0-w/2,0-d/2,0], [w-w/2,0-d/2,0], [w-w/2,d-d/2,0], [0-w/2,d-d/2,0], [0-w/2,0-d/2,0]]
    base = container.add_face pts
    base.material = [170, 170, 170] 
    base.back_material = [170, 170, 170]
    h = -h if base.normal.dot(Z_AXIS) < 0.0
    base.pushpull -h
end

def default_parameters
    defaults = {"w", 100.mm, "d", 100.mm, "h", 3.mm}
    defaults
end

def translate_key(key)
    prompt = key
    case( key )
        when "w"
            prompt = "Width"
        when "h"
            prompt = "Wall"
        when "d"
            prompt = "Depth"
    end
    prompt
end

end

#=============================================================================

class Cylinder < Parametric

def create_entities(data, container)

    r = data["r"].to_l
    u = data["u"].to_l
    n = 24

    circle = container.add_circle ORIGIN, Z_AXIS, r, n
    base = container.add_face circle
    base.material = [170, 170, 170] 
    base.back_material = [170, 170, 170]
    u = -u if base.normal.dot(Z_AXIS) < 0.0
    base.pushpull -u
    
end

def default_parameters
    defaults = {"r", 20.mm, "u", 3.mm}
    defaults
end

def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "u"
            prompt = "Wall"
    end
    prompt
end

end

#=============================================================================

class Prism < Parametric

def create_entities(data, container)

    r = data["r"].to_l
    u = data["u"].to_l
    s = data["s"]

    circle = container.add_ngon ORIGIN, Z_AXIS, r, s
    base = container.add_face circle
    base.material = [170, 170, 170] 
    base.back_material = [170, 170, 170]
    u = -u if base.normal.dot(Z_AXIS) < 0.0
    base.pushpull u
    
end

def default_parameters
    defaults = {"r", 20.mm, "s", 6, "u", 100.mm}
    defaults
end


def validate_parameters(data)
    ok = true
        if( data["s"] < 3 )
        UI.messagebox "At least 3 sides required"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "u"
            prompt = "Height"
        when "s"
            prompt = "Number of Sides"
    end
    prompt
end

end

#=============================================================================

class Cone < Parametric

def create_entities(data, container)

    r = data["r"].to_l
    u = data["u"].to_l
    n = 24

    # Create the base
    circle = container.add_circle ORIGIN, Z_AXIS, r, n
    base = container.add_face circle
    base.material = [170, 170, 170] 
    base.back_material = [170, 170, 170]
    base_edges = base.edges 
    
    # Create the sides
    apex = [0,0,u]
    e1 = nil
    e2 = nil
    base_edges.each do |edge|
        e2 = container.add_line edge.start.position, apex
        e2.soft = true
        e2.smooth = true
        if( e1 )
        face = container.add_face edge, e2, e1
        face.material = [170, 170, 170] 
        face.back_material = [170, 170, 170]
        end
        e1 = e2
    end
    
    # Create the last side face
    edge = base_edges[0]
    faceend = container.add_face edge.start.position, edge.end.position, apex
    faceend.material = [170, 170, 170] 
    faceend.back_material = [170, 170, 170]
end

def default_parameters
    defaults = {"r", 20.mm, "u", 100.mm}
    defaults
end

def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "u"
            prompt = "Height"
    end
    prompt
end

end

#=============================================================================

class Tube < Parametric

def create_entities(data, container)

    ro = data["r"].to_l
    ri = ro - data["t"].to_l
    u = data["u"].to_l
    n = 24

    outer = container.add_circle ORIGIN, Z_AXIS, ro, n
    face = container.add_face outer
    face.material = [170, 170, 170] 
    face.back_material = [170, 170, 170]
    inner = container.add_circle ORIGIN, Z_AXIS, ri, n
    inner[0].faces.each {|f| f.erase! if( f != face)}
    u = -u if face.normal.dot(Z_AXIS) < 0.0
    face.pushpull u
    
end

def default_parameters
    defaults = {"r", 20.mm, "t", 3.mm, "u", 100.mm}
    defaults
end

def validate_parameters(data)
    ok = true
    if( data["t"] >= data["r"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end

def translate_key(key)
    prompt = key
    case( key )
        when "r"
            prompt = "Radius"
        when "t"
            prompt = "Wall"
        when "u"
            prompt = "Length"
    end
    prompt
end

end

#=============================================================================

class Pyramid < Parametric

def create_entities(data, container)

    b = data["b"].to_l
    h = data["h"].to_l

    # Create the base
    b2 = b / 2.0
    pts = []
    pts[0] = [-b2, -b2, 0]
    pts[1] = [b2, -b2, 0]
    pts[2] = [b2, b2, 0]
    pts[3] = [-b2, b2, 0]
    base = container.add_face pts
    
    # Create the sides
    apex = [0,0,h]
    face1 = container.add_face pts[0], apex, pts[1]
    face2 = container.add_face pts[1], apex, pts[2]
    face3 = container.add_face pts[2], apex, pts[3]
    face4 = container.add_face pts[3], apex, pts[0]
    
    base.material = [170, 170, 170] 
    base.back_material = [170, 170, 170]
    face1.material = [170, 170, 170] 
    face1.back_material = [170, 170, 170]
    face2.material = [170, 170, 170] 
    face2.back_material = [170, 170, 170]
    face3.material = [170, 170, 170] 
    face3.back_material = [170, 170, 170]
    face4.material = [170, 170, 170] 
    face4.back_material = [170, 170, 170]    
end

def default_parameters
    defaults = {"b", 40.mm, "h", 100.mm}
    defaults
end

def translate_key(key)
    prompt = key
    case( key )
        when "b"
            prompt = "Base"
        when "h"
            prompt = "Height"
    end
    prompt
end

end

#=============================================================================

class Beam < Parametric

def create_entities(data, container)

    a1 = data["a"].to_l 
    b1 = data["b"].to_l
    t = data["t"].to_l
    u = data["u"].to_l
    
    a = a1-4*t
    b = b1-4*t
    
    # compute points
    arcptsl = []
    arcptsh = []
    face = []
    delta = Math::PI/8
 
    for i in 0..4 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina+b/2, 0))
        arcptsh.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina+b/2, -u))
    end
       
    for i in 4..8 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina+b/2, 0))
        arcptsh.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina+b/2, -u))
    end
    
    for i in 8..12 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina-b/2, 0))
        arcptsh.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina-b/2, -u))
    end
    
    for i in 12..16 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina-b/2, 0))
        arcptsh.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina-b/2, -u))
    end
 
    for i in 0..18 do
    face[i] = container.add_face arcptsl[i], arcptsh[i], arcptsh[i+1], arcptsl[i+1]
    end
    
    face[19] = container.add_face arcptsl[19], arcptsh[19], arcptsh[0], arcptsl[0]

    face0edges = []
    face0edges = face[0].edges
    face0edges[0].soft = true
    face0edges[0].smooth = true
    face0edges[2].soft = true
    face0edges[2].smooth = true
    
    face1edges = []
    face1edges = face[1].edges
    face1edges[0].soft = true
    face1edges[0].smooth = true
    face1edges[2].soft = true
    face1edges[2].smooth = true
    
    face2edges = []
    face2edges = face[2].edges
    face2edges[0].soft = true
    face2edges[0].smooth = true
    face2edges[2].soft = true
    face2edges[2].smooth = true
    
    face3edges = []
    face3edges = face[3].edges
    face3edges[0].soft = true
    face3edges[0].smooth = true
    face3edges[2].soft = true
    face3edges[2].smooth = true
    
    face4edges = []
    face4edges = face[4].edges
    face4edges[0].soft = true
    face4edges[0].smooth = true
    face4edges[2].soft = true
    face4edges[2].smooth = true    
    
    face5edges = []
    face5edges = face[5].edges
    face5edges[0].soft = true
    face5edges[0].smooth = true
    face5edges[2].soft = true
    face5edges[2].smooth = true 
    
    face6edges = []
    face6edges = face[6].edges
    face6edges[0].soft = true
    face6edges[0].smooth = true
    face6edges[2].soft = true
    face6edges[2].smooth = true     
    
    face7edges = []
    face7edges = face[7].edges
    face7edges[0].soft = true
    face7edges[0].smooth = true
    face7edges[2].soft = true
    face7edges[2].smooth = true   
    
    face8edges = []
    face8edges = face[8].edges
    face8edges[0].soft = true
    face8edges[0].smooth = true
    face8edges[2].soft = true
    face8edges[2].smooth = true      
    
    face9edges = []
    face9edges = face[9].edges
    face9edges[0].soft = true
    face9edges[0].smooth = true
    face9edges[2].soft = true
    face9edges[2].smooth = true    
    
    face10edges = []
    face10edges = face[10].edges
    face10edges[0].soft = true
    face10edges[0].smooth = true
    face10edges[2].soft = true
    face10edges[2].smooth = true     
    
    face11edges = []
    face11edges = face[11].edges
    face11edges[0].soft = true
    face11edges[0].smooth = true
    face11edges[2].soft = true
    face11edges[2].smooth = true      
    
    face12edges = []
    face12edges = face[12].edges
    face12edges[0].soft = true
    face12edges[0].smooth = true
    face12edges[2].soft = true
    face12edges[2].smooth = true      
    
    face13edges = []
    face13edges = face[13].edges
    face13edges[0].soft = true
    face13edges[0].smooth = true
    face13edges[2].soft = true
    face13edges[2].smooth = true    
    
    face14edges = []
    face14edges = face[14].edges
    face14edges[0].soft = true
    face14edges[0].smooth = true
    face14edges[2].soft = true
    face14edges[2].smooth = true
    
    face15edges = []
    face15edges = face[15].edges
    face15edges[0].soft = true
    face15edges[0].smooth = true
    face15edges[2].soft = true
    face15edges[2].smooth = true
    
    face16edges = []
    face16edges = face[16].edges
    face16edges[0].soft = true
    face16edges[0].smooth = true
    face16edges[2].soft = true
    face16edges[2].smooth = true
    
    face17edges = []
    face17edges = face[17].edges
    face17edges[0].soft = true
    face17edges[0].smooth = true
    face17edges[2].soft = true
    face17edges[2].smooth = true
    
    face18edges = []
    face18edges = face[18].edges
    face18edges[0].soft = true
    face18edges[0].smooth = true
    face18edges[2].soft = true
    face18edges[2].smooth = true
    
    face19edges = []
    face19edges = face[19].edges
    face19edges[0].soft = true
    face19edges[0].smooth = true
    face19edges[2].soft = true
    face19edges[2].smooth = true
   
    for i in 0..19 do
    face[i].material = [170, 170, 170] 
    face[i].back_material = [170, 170, 170]
    end     
         
     basel = container.add_face arcptsl
     baseh = container.add_face arcptsh
     basel.material = [170, 170, 170] 
     basel.back_material = [170, 170, 170]
     baseh.material = [170, 170, 170] 
     baseh.back_material = [170, 170, 170]

end

def default_parameters
    defaults = {"a", 100.mm, "b", 100.mm, "t", 3.mm, "u", 3.mm}
    defaults
end


def validate_parameters(data)
    ok = true
    if( 4*data["t"] >= data["a"] ) or ( 4*data["t"] >= data["b"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Wall"
        when "u"
            prompt = "Length"                       
    end
    prompt
end

end

#=============================================================================
   
class Jackel < Parametric

def create_entities(data, container)

    a = data["a"].to_l 
    b = data["b"].to_l
    t = data["t"].to_l
    u = data["u"].to_l
    v = data["v"].to_l
    w = data["w"].to_l
    
    a = a-4*t
    b = b-4*t
    
    ang = Math::PI/180
    angleu = ang*u
    anglev = ang*v
    sine1 = Math.sin(angleu)
    cose1 = Math.cos(angleu)    
    sine2 = Math.sin(anglev)
    cose2 = Math.cos(anglev)
    
    # compute points
    arcptsl = []
    arcptsh = []
    arcptsli = []
    arcptshi = []
    face = []
    facei = []
    delta = Math::PI/8
     
    for i in 0..4 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina+b/2, (2*t*cosa+a/2)*sine1/cose1))
        arcptsh.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina+b/2, w-(2*t*cosa+a/2)*sine2/cose2))
        arcptsli.push(Geom::Point3d.new(t*cosa+a/2, t*sina+b/2, (t*cosa+a/2)*sine1/cose1))
        arcptshi.push(Geom::Point3d.new(t*cosa+a/2, t*sina+b/2, w-(t*cosa+a/2)*sine2/cose2))
    end
       
    for i in 4..8 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina+b/2, (2*t*cosa-a/2)*sine1/cose1))
        arcptsh.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina+b/2, w-(2*t*cosa-a/2)*sine2/cose2))
        arcptsli.push(Geom::Point3d.new(t*cosa-a/2, t*sina+b/2, (t*cosa-a/2)*sine1/cose1))
        arcptshi.push(Geom::Point3d.new(t*cosa-a/2, t*sina+b/2, w-(t*cosa-a/2)*sine2/cose2))
    end
    
    for i in 8..12 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina-b/2, (2*t*cosa-a/2)*sine1/cose1))
        arcptsh.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina-b/2, w-(2*t*cosa-a/2)*sine2/cose2))
        arcptsli.push(Geom::Point3d.new(t*cosa-a/2, t*sina-b/2, (t*cosa-a/2)*sine1/cose1))
        arcptshi.push(Geom::Point3d.new(t*cosa-a/2, t*sina-b/2, w-(t*cosa-a/2)*sine2/cose2))
    end
    
    for i in 12..16 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina-b/2, (2*t*cosa+a/2)*sine1/cose1))
        arcptsh.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina-b/2, w-(2*t*cosa+a/2)*sine2/cose2))
        arcptsli.push(Geom::Point3d.new(t*cosa+a/2, t*sina-b/2, (t*cosa+a/2)*sine1/cose1))
        arcptshi.push(Geom::Point3d.new(t*cosa+a/2, t*sina-b/2, w-(t*cosa+a/2)*sine2/cose2))
    end
 
    for i in 0..18 do
    face[i] = container.add_face arcptsl[i], arcptsh[i], arcptsh[i+1], arcptsl[i+1]
    end
    face[19] = container.add_face arcptsl[19], arcptsh[19], arcptsh[0], arcptsl[0]

    for i in 0..18 do
    facei[i] = container.add_face arcptsli[i], arcptshi[i], arcptshi[i+1], arcptsli[i+1]
    end
    facei[19] = container.add_face arcptsli[19], arcptshi[19], arcptshi[0], arcptsli[0]
       
    for i in 0..19 do
    face[i].material = [170, 170, 170] 
    face[i].back_material = [170, 170, 170]
    facei[i].material = [170, 170, 170] 
    facei[i].back_material = [170, 170, 170]
    end
    
    face0 = []       
    face0 = face[0].edges
    face0[0].soft = true
    face0[0].smooth = true
    face0[2].soft = true
    face0[2].smooth = true
    facei0 = []       
    facei0 = facei[0].edges
    facei0[0].soft = true
    facei0[0].smooth = true
    facei0[2].soft = true
    facei0[2].smooth = true
  
    face1 = []       
    face1 = face[1].edges
    face1[0].soft = true
    face1[0].smooth = true
    face1[2].soft = true
    face1[2].smooth = true
    facei1 = []       
    facei1 = facei[1].edges
    facei1[0].soft = true
    facei1[0].smooth = true
    facei1[2].soft = true
    facei1[2].smooth = true
  
    face2 = []       
    face2 = face[2].edges
    face2[0].soft = true
    face2[0].smooth = true
    face2[2].soft = true
    face2[2].smooth = true
    facei2 = []       
    facei2 = facei[2].edges
    facei2[0].soft = true
    facei2[0].smooth = true
    facei2[2].soft = true
    facei2[2].smooth = true
  
    face3 = []       
    face3 = face[3].edges
    face3[0].soft = true
    face3[0].smooth = true
    face3[2].soft = true
    face3[2].smooth = true
    facei3 = []       
    facei3 = facei[3].edges
    facei3[0].soft = true
    facei3[0].smooth = true
    facei3[2].soft = true
    facei3[2].smooth = true
  
    face4 = []       
    face4 = face[4].edges
    face4[0].soft = true
    face4[0].smooth = true
    face4[2].soft = true
    face4[2].smooth = true
    facei4 = []       
    facei4 = facei[4].edges
    facei4[0].soft = true
    facei4[0].smooth = true
    facei4[2].soft = true
    facei4[2].smooth = true
  
    face5 = []       
    face5 = face[5].edges
    face5[0].soft = true
    face5[0].smooth = true
    face5[2].soft = true
    face5[2].smooth = true
    facei5 = []       
    facei5 = facei[5].edges
    facei5[0].soft = true
    facei5[0].smooth = true
    facei5[2].soft = true
    facei5[2].smooth = true
  
    face6 = []       
    face6 = face[6].edges
    face6[0].soft = true
    face6[0].smooth = true
    face6[2].soft = true
    face6[2].smooth = true
    facei6 = []       
    facei6 = facei[6].edges
    facei6[0].soft = true
    facei6[0].smooth = true
    facei6[2].soft = true
    facei6[2].smooth = true
  
    face7 = []       
    face7 = face[7].edges
    face7[0].soft = true
    face7[0].smooth = true
    face7[2].soft = true
    face7[2].smooth = true
    facei7 = []       
    facei7 = facei[7].edges
    facei7[0].soft = true
    facei7[0].smooth = true
    facei7[2].soft = true
    facei7[2].smooth = true
  
    face8 = []       
    face8 = face[8].edges
    face8[0].soft = true
    face8[0].smooth = true
    face8[2].soft = true
    face8[2].smooth = true
    facei8 = []       
    facei8 = facei[8].edges
    facei8[0].soft = true
    facei8[0].smooth = true
    facei8[2].soft = true
    facei8[2].smooth = true
  
    face9 = []       
    face9 = face[9].edges
    face9[0].soft = true
    face9[0].smooth = true
    face9[2].soft = true
    face9[2].smooth = true
    facei9 = []       
    facei9 = facei[9].edges
    facei9[0].soft = true
    facei9[0].smooth = true
    facei9[2].soft = true
    facei9[2].smooth = true
  
    face10 = []       
    face10 = face[10].edges
    face10[0].soft = true
    face10[0].smooth = true
    face10[2].soft = true
    face10[2].smooth = true
    facei10 = []       
    facei10 = facei[10].edges
    facei10[0].soft = true
    facei10[0].smooth = true
    facei10[2].soft = true
    facei10[2].smooth = true
  
    face11 = []       
    face11 = face[11].edges
    face11[0].soft = true
    face11[0].smooth = true
    face11[2].soft = true
    face11[2].smooth = true
    facei11 = []       
    facei11 = facei[11].edges
    facei11[0].soft = true
    facei11[0].smooth = true
    facei11[2].soft = true
    facei11[2].smooth = true
  
    face12 = []       
    face12 = face[12].edges
    face12[0].soft = true
    face12[0].smooth = true
    face12[2].soft = true
    face12[2].smooth = true
    facei12 = []       
    facei12 = facei[12].edges
    facei12[0].soft = true
    facei12[0].smooth = true
    facei12[2].soft = true
    facei12[2].smooth = true
  
    face13 = []       
    face13 = face[13].edges
    face13[0].soft = true
    face13[0].smooth = true
    face13[2].soft = true
    face13[2].smooth = true
    facei13 = []       
    facei13 = facei[13].edges
    facei13[0].soft = true
    facei13[0].smooth = true
    facei13[2].soft = true
    facei13[2].smooth = true
  
    face14 = []       
    face14 = face[14].edges
    face14[0].soft = true
    face14[0].smooth = true
    face14[2].soft = true
    face14[2].smooth = true
    facei14 = []       
    facei14 = facei[14].edges
    facei14[0].soft = true
    facei14[0].smooth = true
    facei14[2].soft = true
    facei14[2].smooth = true
  
    face15 = []       
    face15 = face[15].edges
    face15[0].soft = true
    face15[0].smooth = true
    face15[2].soft = true
    face15[2].smooth = true
    facei15 = []       
    facei15 = facei[15].edges
    facei15[0].soft = true
    facei15[0].smooth = true
    facei15[2].soft = true
    facei15[2].smooth = true
  
    face16 = []       
    face16 = face[16].edges
    face16[0].soft = true
    face16[0].smooth = true
    face16[2].soft = true
    face16[2].smooth = true
    facei16 = []       
    facei16 = facei[16].edges
    facei16[0].soft = true
    facei16[0].smooth = true
    facei16[2].soft = true
    facei16[2].smooth = true
  
    face17 = []       
    face17 = face[17].edges
    face17[0].soft = true
    face17[0].smooth = true
    face17[2].soft = true
    face17[2].smooth = true
    facei17 = []       
    facei17 = facei[17].edges
    facei17[0].soft = true
    facei17[0].smooth = true
    facei17[2].soft = true
    facei17[2].smooth = true
  
    face18 = []       
    face18 = face[18].edges
    face18[0].soft = true
    face18[0].smooth = true
    face18[2].soft = true
    face18[2].smooth = true
    facei18 = []       
    facei18 = facei[18].edges
    facei18[0].soft = true
    facei18[0].smooth = true
    facei18[2].soft = true
    facei18[2].smooth = true
  
    face19 = []       
    face19 = face[19].edges
    face19[0].soft = true
    face19[0].smooth = true
    face19[2].soft = true
    face19[2].smooth = true
    facei19 = []       
    facei19 = facei[19].edges
    facei19[0].soft = true
    facei19[0].smooth = true
    facei19[2].soft = true
    facei19[2].smooth = true
        
    basel = container.add_face arcptsl
    baseli = container.add_face arcptsli
    baseh = container.add_face arcptsh
    basehi = container.add_face arcptshi
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]
    baseli.erase!
    basehi.erase!

end

def default_parameters
    defaults = {"a", 100.mm, "b", 100.mm, "t", 3.mm, "u", 0.0, "v", 0.0, "w", 500.mm}
    defaults
end

def validate_parameters(data)
    ok = true
    if( 4*data["t"] >= data["a"] ) or ( 4*data["t"] >= data["b"] ) 
        UI.messagebox "Thickness must be smaller"
        ok = false
    end

    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Wall"
        when "u"
            prompt = "End Angle 1"
        when "v"
            prompt = "End Angle 2"                  
        when "w"
            prompt = "Length"                       
    end
    prompt
end

end
#=============================================================================

class Jackel_diam < Parametric

def create_entities(data, container)

    a = data["a"].to_l 
    t = data["t"].to_l
    u = data["u"].to_l
    v = data["v"].to_l
    w = data["w"].to_l
    
    ang = Math::PI/180
    angleu = ang*u
    anglev = ang*v
    sine1 = Math.sin(angleu)
    cose1 = Math.cos(angleu)    
    sine2 = Math.sin(anglev)
    cose2 = Math.cos(anglev)
    anglek = ang*45
    
    a = (a-4*t)/Math.cos(anglek)
    
    # compute points
    arcptsl = []
    arcptsh = []
    arcptsli = []
    arcptshi = []
    face = []
    facei = []
    delta = Math::PI/8
    
    for i in -2..2 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina, (2*t*cosa+a/2)*sine1/cose1))
        arcptsh.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina, w-(2*t*cosa+a/2)*sine2/cose2))
        arcptsli.push(Geom::Point3d.new(t*cosa+a/2, t*sina, (t*cosa+a/2)*sine1/cose1))
        arcptshi.push(Geom::Point3d.new(t*cosa+a/2, t*sina, w-(t*cosa+a/2)*sine2/cose2))
    end
       
    for i in 2..6 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa, 2*t*sina+a/2, (2*t*cosa)*sine1/cose1))
        arcptsh.push(Geom::Point3d.new(2*t*cosa, 2*t*sina+a/2, w-(2*t*cosa)*sine2/cose2))
        arcptsli.push(Geom::Point3d.new(t*cosa, t*sina+a/2, (t*cosa)*sine1/cose1))
        arcptshi.push(Geom::Point3d.new(t*cosa, t*sina+a/2, w-(t*cosa)*sine2/cose2))
    end
    
    for i in 6..10 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina, (2*t*cosa-a/2)*sine1/cose1))
        arcptsh.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina, w-(2*t*cosa-a/2)*sine2/cose2))
        arcptsli.push(Geom::Point3d.new(t*cosa-a/2, t*sina, (t*cosa-a/2)*sine1/cose1))
        arcptshi.push(Geom::Point3d.new(t*cosa-a/2, t*sina, w-(t*cosa-a/2)*sine2/cose2))
    end
    
    for i in 10..14 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa, 2*t*sina-a/2, (2*t*cosa)*sine1/cose1))
        arcptsh.push(Geom::Point3d.new(2*t*cosa, 2*t*sina-a/2, w-(2*t*cosa)*sine2/cose2))
        arcptsli.push(Geom::Point3d.new(t*cosa, t*sina-a/2, (t*cosa)*sine1/cose1))
        arcptshi.push(Geom::Point3d.new(t*cosa, t*sina-a/2, w-(t*cosa)*sine2/cose2))
    end
 
    for i in 0..18 do
    face[i] = container.add_face arcptsl[i], arcptsh[i], arcptsh[i+1], arcptsl[i+1]
    end
    face[19] = container.add_face arcptsl[19], arcptsh[19], arcptsh[0], arcptsl[0]

    for i in 0..18 do
    facei[i] = container.add_face arcptsli[i], arcptshi[i], arcptshi[i+1], arcptsli[i+1]
    end
    facei[19] = container.add_face arcptsli[19], arcptshi[19], arcptshi[0], arcptsli[0]
       
    for i in 0..19 do
    face[i].material = [170, 170, 170] 
    face[i].back_material = [170, 170, 170]
    facei[i].material = [170, 170, 170] 
    facei[i].back_material = [170, 170, 170]
    end
    
    face0 = []       
    face0 = face[0].edges
    face0[0].soft = true
    face0[0].smooth = true
    face0[2].soft = true
    face0[2].smooth = true
    facei0 = []       
    facei0 = facei[0].edges
    facei0[0].soft = true
    facei0[0].smooth = true
    facei0[2].soft = true
    facei0[2].smooth = true
  
    face1 = []       
    face1 = face[1].edges
    face1[0].soft = true
    face1[0].smooth = true
    face1[2].soft = true
    face1[2].smooth = true
    facei1 = []       
    facei1 = facei[1].edges
    facei1[0].soft = true
    facei1[0].smooth = true
    facei1[2].soft = true
    facei1[2].smooth = true
  
    face2 = []       
    face2 = face[2].edges
    face2[0].soft = true
    face2[0].smooth = true
    face2[2].soft = true
    face2[2].smooth = true
    facei2 = []       
    facei2 = facei[2].edges
    facei2[0].soft = true
    facei2[0].smooth = true
    facei2[2].soft = true
    facei2[2].smooth = true
  
    face3 = []       
    face3 = face[3].edges
    face3[0].soft = true
    face3[0].smooth = true
    face3[2].soft = true
    face3[2].smooth = true
    facei3 = []       
    facei3 = facei[3].edges
    facei3[0].soft = true
    facei3[0].smooth = true
    facei3[2].soft = true
    facei3[2].smooth = true
  
    face4 = []       
    face4 = face[4].edges
    face4[0].soft = true
    face4[0].smooth = true
    face4[2].soft = true
    face4[2].smooth = true
    facei4 = []       
    facei4 = facei[4].edges
    facei4[0].soft = true
    facei4[0].smooth = true
    facei4[2].soft = true
    facei4[2].smooth = true
  
    face5 = []       
    face5 = face[5].edges
    face5[0].soft = true
    face5[0].smooth = true
    face5[2].soft = true
    face5[2].smooth = true
    facei5 = []       
    facei5 = facei[5].edges
    facei5[0].soft = true
    facei5[0].smooth = true
    facei5[2].soft = true
    facei5[2].smooth = true
  
    face6 = []       
    face6 = face[6].edges
    face6[0].soft = true
    face6[0].smooth = true
    face6[2].soft = true
    face6[2].smooth = true
    facei6 = []       
    facei6 = facei[6].edges
    facei6[0].soft = true
    facei6[0].smooth = true
    facei6[2].soft = true
    facei6[2].smooth = true
  
    face7 = []       
    face7 = face[7].edges
    face7[0].soft = true
    face7[0].smooth = true
    face7[2].soft = true
    face7[2].smooth = true
    facei7 = []       
    facei7 = facei[7].edges
    facei7[0].soft = true
    facei7[0].smooth = true
    facei7[2].soft = true
    facei7[2].smooth = true
  
    face8 = []       
    face8 = face[8].edges
    face8[0].soft = true
    face8[0].smooth = true
    face8[2].soft = true
    face8[2].smooth = true
    facei8 = []       
    facei8 = facei[8].edges
    facei8[0].soft = true
    facei8[0].smooth = true
    facei8[2].soft = true
    facei8[2].smooth = true
  
    face9 = []       
    face9 = face[9].edges
    face9[0].soft = true
    face9[0].smooth = true
    face9[2].soft = true
    face9[2].smooth = true
    facei9 = []       
    facei9 = facei[9].edges
    facei9[0].soft = true
    facei9[0].smooth = true
    facei9[2].soft = true
    facei9[2].smooth = true
  
    face10 = []       
    face10 = face[10].edges
    face10[0].soft = true
    face10[0].smooth = true
    face10[2].soft = true
    face10[2].smooth = true
    facei10 = []       
    facei10 = facei[10].edges
    facei10[0].soft = true
    facei10[0].smooth = true
    facei10[2].soft = true
    facei10[2].smooth = true
  
    face11 = []       
    face11 = face[11].edges
    face11[0].soft = true
    face11[0].smooth = true
    face11[2].soft = true
    face11[2].smooth = true
    facei11 = []       
    facei11 = facei[11].edges
    facei11[0].soft = true
    facei11[0].smooth = true
    facei11[2].soft = true
    facei11[2].smooth = true
  
    face12 = []       
    face12 = face[12].edges
    face12[0].soft = true
    face12[0].smooth = true
    face12[2].soft = true
    face12[2].smooth = true
    facei12 = []       
    facei12 = facei[12].edges
    facei12[0].soft = true
    facei12[0].smooth = true
    facei12[2].soft = true
    facei12[2].smooth = true
  
    face13 = []       
    face13 = face[13].edges
    face13[0].soft = true
    face13[0].smooth = true
    face13[2].soft = true
    face13[2].smooth = true
    facei13 = []       
    facei13 = facei[13].edges
    facei13[0].soft = true
    facei13[0].smooth = true
    facei13[2].soft = true
    facei13[2].smooth = true
  
    face14 = []       
    face14 = face[14].edges
    face14[0].soft = true
    face14[0].smooth = true
    face14[2].soft = true
    face14[2].smooth = true
    facei14 = []       
    facei14 = facei[14].edges
    facei14[0].soft = true
    facei14[0].smooth = true
    facei14[2].soft = true
    facei14[2].smooth = true
  
    face15 = []       
    face15 = face[15].edges
    face15[0].soft = true
    face15[0].smooth = true
    face15[2].soft = true
    face15[2].smooth = true
    facei15 = []       
    facei15 = facei[15].edges
    facei15[0].soft = true
    facei15[0].smooth = true
    facei15[2].soft = true
    facei15[2].smooth = true
  
    face16 = []       
    face16 = face[16].edges
    face16[0].soft = true
    face16[0].smooth = true
    face16[2].soft = true
    face16[2].smooth = true
    facei16 = []       
    facei16 = facei[16].edges
    facei16[0].soft = true
    facei16[0].smooth = true
    facei16[2].soft = true
    facei16[2].smooth = true
  
    face17 = []       
    face17 = face[17].edges
    face17[0].soft = true
    face17[0].smooth = true
    face17[2].soft = true
    face17[2].smooth = true
    facei17 = []       
    facei17 = facei[17].edges
    facei17[0].soft = true
    facei17[0].smooth = true
    facei17[2].soft = true
    facei17[2].smooth = true
  
    face18 = []       
    face18 = face[18].edges
    face18[0].soft = true
    face18[0].smooth = true
    face18[2].soft = true
    face18[2].smooth = true
    facei18 = []       
    facei18 = facei[18].edges
    facei18[0].soft = true
    facei18[0].smooth = true
    facei18[2].soft = true
    facei18[2].smooth = true
  
    face19 = []       
    face19 = face[19].edges
    face19[0].soft = true
    face19[0].smooth = true
    face19[2].soft = true
    face19[2].smooth = true
    facei19 = []       
    facei19 = facei[19].edges
    facei19[0].soft = true
    facei19[0].smooth = true
    facei19[2].soft = true
    facei19[2].smooth = true
        
    basel = container.add_face arcptsl
    baseli = container.add_face arcptsli
    baseh = container.add_face arcptsh
    basehi = container.add_face arcptshi
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]
    baseli.erase!
    basehi.erase!

end

def default_parameters
    defaults = {"a", 100.mm, "t", 3.mm, "u", 0.0, "v", 0.0, "w", 500.mm}
    defaults
end


def validate_parameters(data)
    ok = true
    if( 4*data["t"] >= data["a"] )
        UI.messagebox "Thickness must be smaller"
        ok = false    
    end

    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "t"
            prompt = "Wall"
        when "u"
            prompt = "End Angle 1"
        when "v"
            prompt = "End Angle 2"                  
        when "w"
            prompt = "Length"                       
    end
    prompt
end

end

#=============================================================================

class U_profile < Parametric

def create_entities(data, container)

    a1 = data["a"].to_l 
    b1 = data["b"].to_l
    t = data["t"].to_l
    u = data["u"].to_l
    
    a = a1-4*t
    b = b1-4*t
    
    # compute points
    arcptsl = []
    arcptsh = []
    arcptsli = []
    arcptshi = []
    face = []
    facei = []
    delta = Math::PI/8
 
    for i in 0..4 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina+b/2, 0))
        arcptsh.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina+b/2, u))
        arcptsli.push(Geom::Point3d.new(t*cosa+a/2, t*sina+b/2, 0))
        arcptshi.push(Geom::Point3d.new(t*cosa+a/2, t*sina+b/2, u))
    end
       
    for i in 4..8 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina+b/2, 0))
        arcptsh.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina+b/2, u))
        arcptsli.push(Geom::Point3d.new(t*cosa-a/2, t*sina+b/2, 0))
        arcptshi.push(Geom::Point3d.new(t*cosa-a/2, t*sina+b/2, u))
    end

        arcptsl.push(Geom::Point3d.new(a/2+2*t, -b/2-2*t, 0))
        arcptsh.push(Geom::Point3d.new(a/2+2*t, -b/2-2*t, u))
        arcptsli.push(Geom::Point3d.new(a/2+t, -b/2-2*t, 0))
        arcptshi.push(Geom::Point3d.new(a/2+t, -b/2-2*t, u))

        arcptsl.push(Geom::Point3d.new(-a/2-2*t, -b/2-2*t, 0))
        arcptsh.push(Geom::Point3d.new(-a/2-2*t, -b/2-2*t, u))
        arcptsli.push(Geom::Point3d.new(-a/2-t, -b/2-2*t, 0))
        arcptshi.push(Geom::Point3d.new(-a/2-t, -b/2-2*t, u))
     
    for i in 0..8 do
    face[i] = container.add_face arcptsl[i], arcptsh[i], arcptsh[i+1], arcptsl[i+1]
    end
   
    for i in 0..8 do
    facei[i] = container.add_face arcptsli[i], arcptshi[i], arcptshi[i+1], arcptsli[i+1]
    end

    face[9] = container.add_face arcptsl[10], arcptsh[10], arcptsh[0], arcptsl[0]
    facei[9] = container.add_face arcptsli[10], arcptshi[10], arcptshi[0], arcptsli[0]
    face[10] = container.add_face arcptsl[9], arcptsh[9], arcptsh[11], arcptsl[11]
    facei[10] = container.add_face arcptsli[9], arcptshi[9], arcptshi[11], arcptsli[11]     
    face[11] = container.add_face arcptsli[10], arcptshi[10], arcptsh[10], arcptsl[10]
    facei[11] = container.add_face arcptsli[11], arcptshi[11], arcptsh[11], arcptsl[11]
    
       
    for i in 0..11 do
    face[i].material = [170, 170, 170] 
    face[i].back_material = [170, 170, 170]
    facei[i].material = [170, 170, 170] 
    facei[i].back_material = [170, 170, 170]
    end
    
    face0 = []       
    face0 = face[0].edges
    face0[0].soft = true
    face0[0].smooth = true
    face0[2].soft = true
    face0[2].smooth = true
    facei0 = []       
    facei0 = facei[0].edges
    facei0[0].soft = true
    facei0[0].smooth = true
    facei0[2].soft = true
    facei0[2].smooth = true
  
    face1 = []       
    face1 = face[1].edges
    face1[0].soft = true
    face1[0].smooth = true
    face1[2].soft = true
    face1[2].smooth = true
    facei1 = []       
    facei1 = facei[1].edges
    facei1[0].soft = true
    facei1[0].smooth = true
    facei1[2].soft = true
    facei1[2].smooth = true
  
    face2 = []       
    face2 = face[2].edges
    face2[0].soft = true
    face2[0].smooth = true
    face2[2].soft = true
    face2[2].smooth = true
    facei2 = []       
    facei2 = facei[2].edges
    facei2[0].soft = true
    facei2[0].smooth = true
    facei2[2].soft = true
    facei2[2].smooth = true
  
    face3 = []       
    face3 = face[3].edges
    face3[0].soft = true
    face3[0].smooth = true
    face3[2].soft = true
    face3[2].smooth = true
    facei3 = []       
    facei3 = facei[3].edges
    facei3[0].soft = true
    facei3[0].smooth = true
    facei3[2].soft = true
    facei3[2].smooth = true
  
    face4 = []       
    face4 = face[4].edges
    face4[0].soft = true
    face4[0].smooth = true
    face4[2].soft = true
    face4[2].smooth = true
    facei4 = []       
    facei4 = facei[4].edges
    facei4[0].soft = true
    facei4[0].smooth = true
    facei4[2].soft = true
    facei4[2].smooth = true
  
    face5 = []       
    face5 = face[5].edges
    face5[0].soft = true
    face5[0].smooth = true
    face5[2].soft = true
    face5[2].smooth = true
    facei5 = []       
    facei5 = facei[5].edges
    facei5[0].soft = true
    facei5[0].smooth = true
    facei5[2].soft = true
    facei5[2].smooth = true
  
    face6 = []       
    face6 = face[6].edges
    face6[0].soft = true
    face6[0].smooth = true
    face6[2].soft = true
    face6[2].smooth = true
    facei6 = []       
    facei6 = facei[6].edges
    facei6[0].soft = true
    facei6[0].smooth = true
    facei6[2].soft = true
    facei6[2].smooth = true
  
    face7 = []       
    face7 = face[7].edges
    face7[0].soft = true
    face7[0].smooth = true
    face7[2].soft = true
    face7[2].smooth = true
    facei7 = []       
    facei7 = facei[7].edges
    facei7[0].soft = true
    facei7[0].smooth = true
    facei7[2].soft = true
    facei7[2].smooth = true
  
    face8 = []       
    face8 = face[8].edges
    face8[0].soft = true
    face8[0].smooth = true
    face8[2].soft = true
    face8[2].smooth = true
    facei8 = []       
    facei8 = facei[8].edges
    facei8[0].soft = true
    facei8[0].smooth = true
    facei8[2].soft = true
    facei8[2].smooth = true
        
    basel = container.add_face arcptsl[10], arcptsl[0], arcptsl[1], arcptsl[2], arcptsl[3], arcptsl[4], arcptsl[5], arcptsl[6], arcptsl[7], arcptsl[8], arcptsl[9], arcptsl[11], arcptsli[11], arcptsli[9], arcptsli[8], arcptsli[7], arcptsli[6], arcptsli[5], arcptsli[4], arcptsli[3], arcptsli[2], arcptsli[1], arcptsli[0], arcptsli[10]
    baseh = container.add_face arcptsh[10], arcptsh[0], arcptsh[1], arcptsh[2], arcptsh[3], arcptsh[4], arcptsh[5], arcptsh[6], arcptsh[7], arcptsh[8], arcptsh[9], arcptsh[11], arcptshi[11], arcptshi[9], arcptshi[8], arcptshi[7], arcptshi[6], arcptshi[5], arcptshi[4], arcptshi[3], arcptshi[2], arcptshi[1], arcptshi[0], arcptshi[10]

    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]
    baseli.erase!
    basehi.erase!

end

def default_parameters
    defaults = {"a", 100.mm, "b", 50.mm, "t", 3.mm, "u", 500.mm}
    defaults
end

def validate_parameters(data)
    ok = true
    if( 4*data["t"] >= data["a"] ) or ( 2*data["t"] >= data["b"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Wall"
        when "u"
            prompt = "Height"                       
    end
    prompt
end

end


#=============================================================================

class Z_profile < Parametric

def create_entities(data, container)

    a = data["a"].to_l 
    b = data["b"].to_l
    t = data["t"].to_l
    u = data["u"].to_l
    
    # compute points
    pointl = []
    pointh = []
    face = []
    line = []
    
    angle22 = Math::PI*1/8
    angle45 = Math::PI*2/8
    angle67 = Math::PI*3/8
        
    cos22 = Math.cos(angle22)
    cos45 = Math.cos(angle45)
    cos67 = Math.cos(angle67)

    sin22 = Math.sin(angle22)
    sin45 = Math.sin(angle45)
    sin67 = Math.sin(angle67)

        
pointl.push(Geom::Point3d.new(-a/2, b-t/2, 0))
pointh.push(Geom::Point3d.new(-a/2, b-t/2, u))
pointl.push(Geom::Point3d.new(-a/2+t, b-t/2, 0))
pointh.push(Geom::Point3d.new(-a/2+t, b-t/2, u))
pointl.push(Geom::Point3d.new(-a/2+t, t*3/2, 0))
pointh.push(Geom::Point3d.new(-a/2+t, t*3/2, u))
pointl.push(Geom::Point3d.new(-a/2+2*t-t*cos22, t*3/2-t*sin22, 0))
pointh.push(Geom::Point3d.new(-a/2+2*t-t*cos22, t*3/2-t*sin22, u))
pointl.push(Geom::Point3d.new(-a/2+2*t-t*cos45, t*3/2-t*sin45, 0))
pointh.push(Geom::Point3d.new(-a/2+2*t-t*cos45, t*3/2-t*sin45, u))
pointl.push(Geom::Point3d.new(-a/2+2*t-t*cos67, t*3/2-t*sin67, 0))
pointh.push(Geom::Point3d.new(-a/2+2*t-t*cos67, t*3/2-t*sin67, u))
pointl.push(Geom::Point3d.new(-a/2+2*t, t*1/2, 0))
pointh.push(Geom::Point3d.new(-a/2+2*t, t*1/2, u))        
pointl.push(Geom::Point3d.new(a/2-2*t, t*1/2, 0))
pointh.push(Geom::Point3d.new(a/2-2*t, t*1/2, u))
pointl.push(Geom::Point3d.new(a/2-2*t+2*t*sin22, t*1/2-2*t+2*t*cos22, 0))
pointh.push(Geom::Point3d.new(a/2-2*t+2*t*sin22, t*1/2-2*t+2*t*cos22, u))
pointl.push(Geom::Point3d.new(a/2-2*t+2*t*sin45, t*1/2-2*t+2*t*cos45, 0))
pointh.push(Geom::Point3d.new(a/2-2*t+2*t*sin45, t*1/2-2*t+2*t*cos45, u)) 
pointl.push(Geom::Point3d.new(a/2-2*t+2*t*sin67, t*1/2-2*t+2*t*cos67, 0))
pointh.push(Geom::Point3d.new(a/2-2*t+2*t*sin67, t*1/2-2*t+2*t*cos67, u))
pointl.push(Geom::Point3d.new(a/2, t*1/2-2*t, 0))
pointh.push(Geom::Point3d.new(a/2, t*1/2-2*t, u))
pointl.push(Geom::Point3d.new(a/2, t*1/2-b, 0))
pointh.push(Geom::Point3d.new(a/2, t*1/2-b, u))
pointl.push(Geom::Point3d.new(a/2-t, t*1/2-b, 0))
pointh.push(Geom::Point3d.new(a/2-t, t*1/2-b, u))
pointl.push(Geom::Point3d.new(a/2-t, -t*3/2, 0))
pointh.push(Geom::Point3d.new(a/2-t, -t*3/2, u))
pointl.push(Geom::Point3d.new(a/2-2*t+t*cos22, -t*3/2+t*sin22, 0))
pointh.push(Geom::Point3d.new(a/2-2*t+t*cos22, -t*3/2+t*sin22, u))
pointl.push(Geom::Point3d.new(a/2-2*t+t*cos45, -t*3/2+t*sin45, 0))
pointh.push(Geom::Point3d.new(a/2-2*t+t*cos45, -t*3/2+t*sin45, u))
pointl.push(Geom::Point3d.new(a/2-2*t+t*cos67, -t*3/2+t*sin67, 0))
pointh.push(Geom::Point3d.new(a/2-2*t+t*cos67, -t*3/2+t*sin67, u))
pointl.push(Geom::Point3d.new(a/2-2*t, -t/2, 0))
pointh.push(Geom::Point3d.new(a/2-2*t, -t/2, u))
pointl.push(Geom::Point3d.new(-a/2+2*t, -t/2, 0))
pointh.push(Geom::Point3d.new(-a/2+2*t, -t/2, u))
pointl.push(Geom::Point3d.new(-a/2+2*t-2*t*sin22, t*3/2-2*t*cos22, 0))
pointh.push(Geom::Point3d.new(-a/2+2*t-2*t*sin22, t*3/2-2*t*cos22, u))
pointl.push(Geom::Point3d.new(-a/2+2*t-2*t*sin45, t*3/2-2*t*cos45, 0))
pointh.push(Geom::Point3d.new(-a/2+2*t-2*t*sin45, t*3/2-2*t*cos45, u))
pointl.push(Geom::Point3d.new(-a/2+2*t-2*t*sin67, t*3/2-2*t*cos67, 0))
pointh.push(Geom::Point3d.new(-a/2+2*t-2*t*sin67, t*3/2-2*t*cos67, u))
pointl.push(Geom::Point3d.new(-a/2, t*3/2, 0))
pointh.push(Geom::Point3d.new(-a/2, t*3/2, u))

for i in 0..22
face[i] = container.add_face pointl[i], pointh[i], pointh[i+1], pointl[i+1]
face[i].material = [170, 170, 170] 
face[i].back_material = [170, 170, 170]
end

face[23] = container.add_face pointl[23], pointh[23], pointh[0], pointl[0]
face[23].material = [170, 170, 170] 
face[23].back_material = [170, 170, 170]

    line0 = []       
    line0 = face[2].edges
    line0[0].soft = true
    line0[0].smooth = true
    line0[2].soft = true
    line0[2].smooth = true
    
    line1 = []       
    line1 = face[4].edges
    line1[0].soft = true
    line1[0].smooth = true
    line1[2].soft = true
    line1[2].smooth = true
    
    line2 = []       
    line2 = face[6].edges
    line2[0].soft = true
    line2[0].smooth = true
    line2[2].soft = true
    line2[2].smooth = true
    
    line3 = []       
    line3 = face[8].edges
    line3[0].soft = true
    line3[0].smooth = true
    line3[2].soft = true
    line3[2].smooth = true
    
    line4 = []       
    line4 = face[10].edges
    line4[0].soft = true
    line4[0].smooth = true
    line4[2].soft = true
    line4[2].smooth = true
    
    line5 = []       
    line5 = face[14].edges
    line5[0].soft = true
    line5[0].smooth = true
    line5[2].soft = true
    line5[2].smooth = true
    
    line6 = []       
    line6 = face[16].edges
    line6[0].soft = true
    line6[0].smooth = true
    line6[2].soft = true
    line6[2].smooth = true
    
    line7 = []       
    line7 = face[18].edges
    line7[0].soft = true
    line7[0].smooth = true
    line7[2].soft = true
    line7[2].smooth = true
    
    line8 = []       
    line8 = face[20].edges
    line8[0].soft = true
    line8[0].smooth = true
    line8[2].soft = true
    line8[2].smooth = true
    
    line9 = []       
    line9 = face[22].edges
    line9[0].soft = true
    line9[0].smooth = true
    line9[2].soft = true
    line9[2].smooth = true        
    
    basel = container.add_face pointl
    baseh = container.add_face pointh
    
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]

end

def default_parameters
    defaults = {"a", 100.mm, "b", 50.mm, "t", 3.mm, "u", 500.mm}
    defaults
end


def validate_parameters(data)
    ok = true
    # make sure that the thickness is less than the width
    if( 4*data["t"] >= data["a"] ) or ( 2*data["t"] >= data["b"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Wall"
        when "u"
            prompt = "Length"                       
    end
    prompt
end

end


#=============================================================================

class L_profile90 < Parametric

def create_entities(data, container)

    a = data["a"].to_l 
    b = data["b"].to_l
    t = data["t"].to_l
    u = data["u"].to_l
    v = data["v"].to_l
    w = data["w"].to_l
    
    # compute points
    pointl = []
    pointh = []
    face = []
    line = []
    
    ang = Math::PI/180
    angleu = ang*u
    anglev = ang*v
    sine1 = Math.sin(angleu)
    cose1 = Math.cos(angleu)    
    sine2 = Math.sin(anglev)
    cose2 = Math.cos(anglev)
    
    angle22 = Math::PI*1/8
    angle45 = Math::PI*2/8
    angle67 = Math::PI*3/8
        
    cos22 = Math.cos(angle22)
    cos45 = Math.cos(angle45)
    cos67 = Math.cos(angle67)

    sin22 = Math.sin(angle22)
    sin45 = Math.sin(angle45)
    sin67 = Math.sin(angle67)
        
pointl.push(Geom::Point3d.new(0, b, 0))
pointh.push(Geom::Point3d.new(0, b, w))
pointl.push(Geom::Point3d.new(t, b, t*sine1/cose1))
pointh.push(Geom::Point3d.new(t, b, w-t*sine2/cose2))
pointl.push(Geom::Point3d.new(t, t*2, t*sine1/cose1))
pointh.push(Geom::Point3d.new(t, t*2, w-t*sine2/cose2))
pointl.push(Geom::Point3d.new(2*t-t*cos22, t*2-t*sin22, (2*t-t*cos22)*sine1/cose1))
pointh.push(Geom::Point3d.new(2*t-t*cos22, t*2-t*sin22, w-(2*t-t*cos22)*sine2/cose2))
pointl.push(Geom::Point3d.new(2*t-t*cos45, t*2-t*sin45, (2*t-t*cos45)*sine1/cose1))
pointh.push(Geom::Point3d.new(2*t-t*cos45, t*2-t*sin45, w-(2*t-t*cos45)*sine2/cose2))
pointl.push(Geom::Point3d.new(2*t-t*cos67, t*2-t*sin67, (2*t-t*cos67)*sine1/cose1))
pointh.push(Geom::Point3d.new(2*t-t*cos67, t*2-t*sin67, w-(2*t-t*cos67)*sine2/cose2))
pointl.push(Geom::Point3d.new(2*t, t, 2*t*sine1/cose1))
pointh.push(Geom::Point3d.new(2*t, t, w-2*t*sine2/cose2))
pointl.push(Geom::Point3d.new(a, t, a*sine1/cose1))
pointh.push(Geom::Point3d.new(a, t, w-a*sine2/cose2))
pointl.push(Geom::Point3d.new(a, 0, a*sine1/cose1))
pointh.push(Geom::Point3d.new(a, 0, w-a*sine2/cose2))
pointl.push(Geom::Point3d.new(2*t, 0, 2*t*sine1/cose1))
pointh.push(Geom::Point3d.new(2*t, 0, w-2*t*sine2/cose2))
pointl.push(Geom::Point3d.new(2*t-2*t*sin22, 2*t-2*t*cos22, (2*t-2*t*sin22)*sine1/cose1))
pointh.push(Geom::Point3d.new(2*t-2*t*sin22, 2*t-2*t*cos22, w-(2*t-2*t*sin22)*sine2/cose2))
pointl.push(Geom::Point3d.new(2*t-2*t*sin45, 2*t-2*t*cos45, (2*t-2*t*sin45)*sine1/cose1))
pointh.push(Geom::Point3d.new(2*t-2*t*sin45, 2*t-2*t*cos45, w-(2*t-2*t*sin45)*sine2/cose2))
pointl.push(Geom::Point3d.new(2*t-2*t*sin67, 2*t-2*t*cos67, (2*t-2*t*sin67)*sine1/cose1))
pointh.push(Geom::Point3d.new(2*t-2*t*sin67, 2*t-2*t*cos67, w-(2*t-2*t*sin67)*sine2/cose2))
pointl.push(Geom::Point3d.new(0, 2*t, 0))
pointh.push(Geom::Point3d.new(0, 2*t, w))

for i in 0..12
face[i] = container.add_face pointl[i], pointh[i], pointh[i+1], pointl[i+1]
face[i].material = [170, 170, 170] 
face[i].back_material = [170, 170, 170]
end

face[13] = container.add_face pointl[13], pointh[13], pointh[0], pointl[0]
face[13].material = [170, 170, 170] 
face[13].back_material = [170, 170, 170]

    line0 = []       
    line0 = face[2].edges
    line0[0].soft = true
    line0[0].smooth = true
    line0[2].soft = true
    line0[2].smooth = true
    
    line1 = []       
    line1 = face[3].edges
    line1[0].soft = true
    line1[0].smooth = true
    line1[2].soft = true
    line1[2].smooth = true
    
    line2 = []       
    line2 = face[5].edges
    line2[0].soft = true
    line2[0].smooth = true
    line2[2].soft = true
    line2[2].smooth = true
    
    line3 = []       
    line3 = face[9].edges
    line3[0].soft = true
    line3[0].smooth = true
    line3[2].soft = true
    line3[2].smooth = true
    
    line4 = []       
    line4 = face[10].edges
    line4[0].soft = true
    line4[0].smooth = true
    line4[2].soft = true
    line4[2].smooth = true
    
    line5 = []       
    line5 = face[12].edges
    line5[0].soft = true
    line5[0].smooth = true
    line5[2].soft = true
    line5[2].smooth = true

    
    basel = container.add_face pointl
    baseh = container.add_face pointh
    
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]

end

def default_parameters
    defaults = {"a", 100.mm, "b", 50.mm, "t", 3.mm, "u", 0.0, "v", 0.0, "w", 500.mm}
    defaults
end


def validate_parameters(data)
    ok = true
    if( 2*data["t"] >= data["a"] ) or ( 2*data["t"] >= data["b"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Wall"
        when "u"
            prompt = "End Angle 1"
        when "v"
            prompt = "End Angle 2"                       
        when "w"
            prompt = "Length "                       
            
                                   
    end
    prompt
end

end
#=============================================================================

class L_flange45 < Parametric

def create_entities(data, container)

    a = data["a"].to_l
    t = data["t"].to_l
    
    # compute points
    pointl = []
    pointh = []
    face = []
    line = []
    
    angle22 = Math::PI*1/8
    angle45 = Math::PI*2/8
    angle67 = Math::PI*3/8
        
    cos22 = Math.cos(angle22)
    cos45 = Math.cos(angle45)
    cos67 = Math.cos(angle67)

    sin22 = Math.sin(angle22)
    sin45 = Math.sin(angle45)
    sin67 = Math.sin(angle67)

        
pointl.push(Geom::Point3d.new(0, a, 0))
pointh.push(Geom::Point3d.new(0, a, a))
pointl.push(Geom::Point3d.new(t, a, 0))
pointh.push(Geom::Point3d.new(t, a, a))
pointl.push(Geom::Point3d.new(t, t*2, 0))
pointh.push(Geom::Point3d.new(t, t*2, a))
pointl.push(Geom::Point3d.new(2*t-t*cos22, 2*t-t*sin22, 0))
pointh.push(Geom::Point3d.new(2*t-t*cos22, 2*t-t*sin22, a))
pointl.push(Geom::Point3d.new(2*t-t*cos45, 2*t-t*sin45, 0))
pointh.push(Geom::Point3d.new(2*t-t*cos45, 2*t-t*sin45, a))
pointl.push(Geom::Point3d.new(a+2*t-t*cos45, 2*t-t*sin45-a, 0))
pointh.push(Geom::Point3d.new(a+2*t-t*cos45, 2*t-t*sin45-a, a))
pointl.push(Geom::Point3d.new(a+2*t-2*t*cos45, 2*t-2*t*sin45-a, 0))
pointh.push(Geom::Point3d.new(a+2*t-2*t*cos45, 2*t-2*t*sin45-a, a))
pointl.push(Geom::Point3d.new(2*t-2*t*sin45, 2*t-2*t*cos45, 0))
pointh.push(Geom::Point3d.new(2*t-2*t*sin45, 2*t-2*t*cos45, a))
pointl.push(Geom::Point3d.new(2*t-2*t*sin67, 2*t-2*t*cos67, 0))
pointh.push(Geom::Point3d.new(2*t-2*t*sin67, 2*t-2*t*cos67, a))
pointl.push(Geom::Point3d.new(0, 2*t, 0))
pointh.push(Geom::Point3d.new(0, 2*t, a))

for i in 0..8
face[i] = container.add_face pointl[i], pointh[i], pointh[i+1], pointl[i+1]
face[i].material = [170, 170, 170] 
face[i].back_material = [170, 170, 170]
end

face[9] = container.add_face pointl[9], pointh[9], pointh[0], pointl[0]
face[9].material = [170, 170, 170] 
face[9].back_material = [170, 170, 170]

    line0 = []       
    line0 = face[2].edges
    line0[0].soft = true
    line0[0].smooth = true
    line0[2].soft = true
    line0[2].smooth = true
    
    line1 = []       
    line1 = face[3].edges
    line1[0].soft = true
    line1[0].smooth = true
    line1[2].soft = true
    line1[2].smooth = true
    
    line2 = []       
    line2 = face[7].edges
    line2[0].soft = true
    line2[0].smooth = true
    line2[2].soft = true
    line2[2].smooth = true
    
    line3 = []       
    line3 = face[8].edges
    line3[0].soft = true
    line3[0].smooth = true
    line3[2].soft = true
    line3[2].smooth = true
    
    
    basel = container.add_face pointl
    baseh = container.add_face pointh
    
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]

pointl.push(Geom::Point3d.new(0, a-a/4, 0))
pointl.push(Geom::Point3d.new(0, a, a/4))
pointh.push(Geom::Point3d.new(0, a-a/4, a))
pointh.push(Geom::Point3d.new(0, a, a-a/4))

pointl.push(Geom::Point3d.new(a+2*t-t*cos45-a/20*cos45, 2*t-t*sin45-a+a/20*cos45, 0))
pointl.push(Geom::Point3d.new(a+2*t-t*cos45, 2*t-t*sin45-a, a/20))

pointh.push(Geom::Point3d.new(a+2*t-t*cos45-a/20*cos45, 2*t-t*sin45-a+a/20*cos45, a))
pointh.push(Geom::Point3d.new(a+2*t-t*cos45, 2*t-t*sin45-a, a-a/20))

point01 = Geom::Point3d.new (0, a/2, a/2)
container.add_cpoint point01
point02 = Geom::Point3d.new (t, a/2, a/2)
container.add_cpoint point02

chamferl = container.add_face pointl[0], pointl[10], pointl[11]
chamferh = container.add_face pointh[0], pointh[10], pointh[11]
chamferl1 = container.add_face pointl[5], pointl[12], pointl[13]
chamferh1 = container.add_face pointh[5], pointh[12], pointh[13]
t = -t if chamferl.normal.dot(X_AXIS) < 0.0
chamferl.pushpull t
chamferh.pushpull t
chamferl1.pushpull t
chamferh1.pushpull t

end

def default_parameters
    defaults = {"a", 100.mm, "t", 5.mm,}
    defaults
end

def validate_parameters(data)
    ok = true
    if( 2*data["t"] >= data["a"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "t"
            prompt = "Wall"                     
    end
    prompt
end

end

#=============================================================================

class L_flange50 < Parametric

def create_entities(data, container)

    a = data["a"].to_l
    t = data["t"].to_l
    
    # compute points
    pointl = []
    pointh = []
    face = []
    line = []
    
    angle22 = Math::PI*1/8
    angle45 = Math::PI*2/8
    angle50 = Math::PI*5/18
    angle67 = Math::PI*3/8
        
    cos22 = Math.cos(angle22)
    cos45 = Math.cos(angle45)
    cos50 = Math.cos(angle50)
    cos67 = Math.cos(angle67)

    sin22 = Math.sin(angle22)
    sin45 = Math.sin(angle45)
    sin67 = Math.sin(angle67)
    
    dif = (a/cos50-a/cos45)*sin45 
        
pointl.push(Geom::Point3d.new(0, a, 0))
pointh.push(Geom::Point3d.new(0, a, a))
pointl.push(Geom::Point3d.new(t, a, 0))
pointh.push(Geom::Point3d.new(t, a, a))
pointl.push(Geom::Point3d.new(t, t*2, 0))
pointh.push(Geom::Point3d.new(t, t*2, a))
pointl.push(Geom::Point3d.new(2*t-t*cos22, 2*t-t*sin22, 0))
pointh.push(Geom::Point3d.new(2*t-t*cos22, 2*t-t*sin22, a))
pointl.push(Geom::Point3d.new(2*t-t*cos45, 2*t-t*sin45, 0))
pointh.push(Geom::Point3d.new(2*t-t*cos45, 2*t-t*sin45, a))
pointl.push(Geom::Point3d.new(a+2*t-t*cos45+dif, 2*t-t*sin45-a-dif, 0))
pointh.push(Geom::Point3d.new(a+2*t-t*cos45+dif, 2*t-t*sin45-a-dif, a))
pointl.push(Geom::Point3d.new(a+2*t-2*t*cos45+dif, 2*t-2*t*sin45-a-dif, 0))
pointh.push(Geom::Point3d.new(a+2*t-2*t*cos45+dif, 2*t-2*t*sin45-a-dif, a))
pointl.push(Geom::Point3d.new(2*t-2*t*sin45, 2*t-2*t*cos45, 0))
pointh.push(Geom::Point3d.new(2*t-2*t*sin45, 2*t-2*t*cos45, a))
pointl.push(Geom::Point3d.new(2*t-2*t*sin67, 2*t-2*t*cos67, 0))
pointh.push(Geom::Point3d.new(2*t-2*t*sin67, 2*t-2*t*cos67, a))
pointl.push(Geom::Point3d.new(0, 2*t, 0))
pointh.push(Geom::Point3d.new(0, 2*t, a))

for i in 0..8
face[i] = container.add_face pointl[i], pointh[i], pointh[i+1], pointl[i+1]
face[i].material = [170, 170, 170] 
face[i].back_material = [170, 170, 170]
end

face[9] = container.add_face pointl[9], pointh[9], pointh[0], pointl[0]
face[9].material = [170, 170, 170] 
face[9].back_material = [170, 170, 170]

    line0 = []       
    line0 = face[2].edges
    line0[0].soft = true
    line0[0].smooth = true
    line0[2].soft = true
    line0[2].smooth = true
    
    line1 = []       
    line1 = face[3].edges
    line1[0].soft = true
    line1[0].smooth = true
    line1[2].soft = true
    line1[2].smooth = true
    
    line2 = []       
    line2 = face[7].edges
    line2[0].soft = true
    line2[0].smooth = true
    line2[2].soft = true
    line2[2].smooth = true
    
    line3 = []       
    line3 = face[8].edges
    line3[0].soft = true
    line3[0].smooth = true
    line3[2].soft = true
    line3[2].smooth = true
    
    
    basel = container.add_face pointl
    baseh = container.add_face pointh
    
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]

pointl.push(Geom::Point3d.new(0, a-a/4, 0))
pointl.push(Geom::Point3d.new(0, a, a/4))
pointh.push(Geom::Point3d.new(0, a-a/4, a))
pointh.push(Geom::Point3d.new(0, a, a-a/4))

pointl.push(Geom::Point3d.new(a+2*t-t*cos45-a/20*cos45+dif, 2*t-t*sin45-a+a/20*cos45-dif, 0))
pointl.push(Geom::Point3d.new(a+2*t-t*cos45+dif, 2*t-t*sin45-a-dif, a/20))

pointh.push(Geom::Point3d.new(a+2*t-t*cos45-a/20*cos45+dif, 2*t-t*sin45-a+a/20*cos45-dif, a))
pointh.push(Geom::Point3d.new(a+2*t-t*cos45+dif, 2*t-t*sin45-a-dif, a-a/20))

point01 = Geom::Point3d.new (0, a/2, a/2)
container.add_cpoint point01
point02 = Geom::Point3d.new (t, a/2, a/2)
container.add_cpoint point02

chamferl = container.add_face pointl[0], pointl[10], pointl[11]
chamferh = container.add_face pointh[0], pointh[10], pointh[11]
chamferl1 = container.add_face pointl[5], pointl[12], pointl[13]
chamferh1 = container.add_face pointh[5], pointh[12], pointh[13]
t = -t if chamferl.normal.dot(X_AXIS) < 0.0
chamferl.pushpull t
chamferh.pushpull t
chamferl1.pushpull t
chamferh1.pushpull t

end

def default_parameters
    defaults = {"a", 100.mm, "t", 5.mm,}
    defaults
end

def validate_parameters(data)
    ok = true
    if( 2*data["t"] >= data["a"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "t"
            prompt = "Wall"                     
    end
    prompt
end

end

#=============================================================================

class Cham_box < Parametric

def create_entities(data, container)

    a = data["a"].to_l 
    b = data["b"].to_l
    t = data["t"].to_l
    u = data["u"].to_l
    
    # compute points
    pointl = []
    pointh = []
    face = []
    
pointl.push(Geom::Point3d.new(t-a/2, 0-b/2, 0))
pointh.push(Geom::Point3d.new(t-a/2, 0-b/2, -u))
pointl.push(Geom::Point3d.new(a-t-a/2, 0-b/2, 0))
pointh.push(Geom::Point3d.new(a-t-a/2, 0-b/2, -u))
pointl.push(Geom::Point3d.new(a-a/2, t-b/2, 0))
pointh.push(Geom::Point3d.new(a-a/2, t-b/2, -u))
pointl.push(Geom::Point3d.new(a-a/2, b-t-b/2, 0))
pointh.push(Geom::Point3d.new(a-a/2, b-t-b/2, -u))
pointl.push(Geom::Point3d.new(a-t-a/2, b-b/2, 0))
pointh.push(Geom::Point3d.new(a-t-a/2, b-b/2, -u))
pointl.push(Geom::Point3d.new(t-a/2, b-b/2, 0))
pointh.push(Geom::Point3d.new(t-a/2, b-b/2, -u))
pointl.push(Geom::Point3d.new(0-a/2, b-t-b/2, 0))
pointh.push(Geom::Point3d.new(0-a/2, b-t-b/2, -u))
pointl.push(Geom::Point3d.new(0-a/2, t-b/2, 0))
pointh.push(Geom::Point3d.new(0-a/2, t-b/2, -u))

for i in 0..6
face[i] = container.add_face pointl[i], pointh[i], pointh[i+1], pointl[i+1]
face[i].material = [170, 170, 170] 
face[i].back_material = [170, 170, 170]
end

face[7] = container.add_face pointl[7], pointh[7], pointh[0], pointl[0]
face[7].material = [170, 170, 170] 
face[7].back_material = [170, 170, 170]
    
    basel = container.add_face pointl
    baseh = container.add_face pointh
    
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]

end

def default_parameters
    defaults = {"a", 100.mm, "b", 100.mm, "t", 10.mm, "u", 3.mm}
    defaults
end


def validate_parameters(data)
    ok = true
    if( 2*data["t"] >= data["a"] ) or ( 2*data["t"] >= data["b"] )
        UI.messagebox "Chamfer must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Chamfer"
        when "u"
            prompt = "Wall"                       
    end
    prompt
end

end

#=============================================================================

class Rib < Parametric

def create_entities(data, container)

    a = data["a"].to_l 
    b = data["b"].to_l
    t = data["t"].to_l
    u = data["u"].to_l
    v = data["v"].to_l
    
    # compute points
    pointl = []
    pointh = []
    face = []

    v = (90-v)/2
    angle = Math::PI*v/180
    sin = Math.sin(angle)
    cos = Math.cos(angle)   
    dx = (b-t)*sin/cos 
    dy = (a-t)*sin/cos
    
pointl.push(Geom::Point3d.new(t, 0, 0))
pointh.push(Geom::Point3d.new(t, 0, u))
pointl.push(Geom::Point3d.new(a, dy, 0))
pointh.push(Geom::Point3d.new(a, dy, u))
pointl.push(Geom::Point3d.new(a, dy+t, 0))
pointh.push(Geom::Point3d.new(a, dy+t, u))
pointl.push(Geom::Point3d.new(dx+t, b, 0))
pointh.push(Geom::Point3d.new(dx+t, b, u))
pointl.push(Geom::Point3d.new(dx, b, 0))
pointh.push(Geom::Point3d.new(dx, b, u))
pointl.push(Geom::Point3d.new(0, t, 0))
pointh.push(Geom::Point3d.new(0, t, u))

for i in 0..4
face[i] = container.add_face pointl[i], pointh[i], pointh[i+1], pointl[i+1]
face[i].material = [170, 170, 170] 
face[i].back_material = [170, 170, 170]
end

face[5] = container.add_face pointl[5], pointh[5], pointh[0], pointl[0]
face[5].material = [170, 170, 170] 
face[5].back_material = [170, 170, 170]
    
    basel = container.add_face pointl
    baseh = container.add_face pointh
    
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]

end

def default_parameters
    defaults = {"a", 100.mm, "b", 100.mm, "t", 10.mm, "u", 5.mm, "v", 90.0}
    defaults
end


def validate_parameters(data)
    ok = true
    if( data["t"] >= data["a"] ) or ( data["t"] >= data["b"] )
        UI.messagebox "Chamfer must be smaller"
        ok = false 
    elsif( data["v"] <= 0 )
        UI.messagebox "Angle must be bigger"
        ok = false
    elsif( data["v"] >= 180 )
        UI.messagebox "Angle must be smaller"        
        ok = false
    elsif( data["t"] <= 0 )
        UI.messagebox "Chamfer must be bigger"        
        ok = false    
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Chamfer"
        when "u"
            prompt = "Wall"
        when "v"
            prompt = "Angle"                                   
    end
    prompt
end

end
#=============================================================================

class L_profile45 < Parametric

def create_entities(data, container)

    a = data["a"].to_l
    b = data["b"].to_l
    t = data["t"].to_l
    v = data["v"].to_l
        
    # compute points
    pointl = []
    pointh = []
    face = []
    line = []
    
    angle22 = Math::PI*1/8        
    cos22 = Math.cos(angle22)
    sin22 = Math.sin(angle22)
       
pointl.push(Geom::Point3d.new(-t*cos22, t*sin22, 0))
pointh.push(Geom::Point3d.new(-t*cos22, t*sin22, v))
pointl.push(Geom::Point3d.new(-t, 0, 0))
pointh.push(Geom::Point3d.new(-t, 0, v))
pointl.push(Geom::Point3d.new(-t*cos22, -t*sin22, 0))
pointh.push(Geom::Point3d.new(-t*cos22, -t*sin22, v))
pointl.push(Geom::Point3d.new(-2*t*cos22, -2*t*sin22, 0))
pointh.push(Geom::Point3d.new(-2*t*cos22, -2*t*sin22, v))
pointl.push(Geom::Point3d.new(-2*t, 0, 0))
pointh.push(Geom::Point3d.new(-2*t, 0, v))
pointl.push(Geom::Point3d.new(-2*t*cos22, 2*t*sin22, 0))
pointh.push(Geom::Point3d.new(-2*t*cos22, 2*t*sin22, v))

for i in 0..4
face[i] = container.add_face pointl[i], pointh[i], pointh[i+1], pointl[i+1]
face[i].material = [170, 170, 170] 
face[i].back_material = [170, 170, 170]
end

face[5] = container.add_face pointl[5], pointh[5], pointh[0], pointl[0]
face[5].material = [170, 170, 170] 
face[5].back_material = [170, 170, 170]

    basel = container.add_face pointl
    baseh = container.add_face pointh
    
    basel.material = [170, 170, 170] 
    basel.back_material = [170, 170, 170]
    baseh.material = [170, 170, 170] 
    baseh.back_material = [170, 170, 170]
        
    face[2].pushpull a
    face[5].pushpull b

    line0 = []       
    line0 = face[0].edges
    line0[0].soft = true
    line0[0].smooth = true
    line0[2].soft = true
    line0[2].smooth = true
    
    line1 = []       
    line1 = face[1].edges
    line1[0].soft = true
    line1[0].smooth = true
    line1[2].soft = true
    line1[2].smooth = true
    
    line2 = []       
    line2 = face[3].edges
    line2[0].soft = true
    line2[0].smooth = true
    line2[2].soft = true
    line2[2].smooth = true
    
    line3 = []       
    line3 = face[4].edges
    line3[0].soft = true
    line3[0].smooth = true
    line3[2].soft = true
    line3[2].smooth = true

end

def default_parameters
    defaults = {"a", 100.mm, "b", 50.mm, "t", 3.mm, "v", 500.mm,}
    defaults
end

def validate_parameters(data)
    ok = true
    if( 2*data["t"] >= data["a"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Wall"
        when "v"
            prompt = "Length"                         
    end
    prompt
end

end

#=============================================================================

class One_Side_Flange < Parametric

def create_entities(data, container)

    a1 = data["a"].to_l 
    b1 = data["b"].to_l
    t = data["t"].to_l
    u = data["u"].to_l
    
    a = a1-4*t
    b = b1-4*t
    
    # compute points
    arcptsl = []
    arcptsh = []
    face = []
    delta = Math::PI/8
 
    for i in 0..4 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina+b/2, 0))
        arcptsh.push(Geom::Point3d.new(2*t*cosa+a/2, 2*t*sina+b/2, -u))
    end
       
    for i in 4..8 do
        angle = delta * i
        cosa = Math.cos(angle)
        sina = Math.sin(angle)
        arcptsl.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina+b/2, 0))
        arcptsh.push(Geom::Point3d.new(2*t*cosa-a/2, 2*t*sina+b/2, -u))
    end

        arcptsl.push(Geom::Point3d.new(-a1/2, -b1/2-a1+a1/4, 0))
        arcptsh.push(Geom::Point3d.new(-a1/2, -b1/2-a1+a1/4, -u))
        arcptsl.push(Geom::Point3d.new(-a1/2+a1/4, -b1/2-a1, 0))
        arcptsh.push(Geom::Point3d.new(-a1/2+a1/4, -b1/2-a1, -u))
        arcptsl.push(Geom::Point3d.new(a1/2-a1/4, -b1/2-a1, 0))
        arcptsh.push(Geom::Point3d.new(a1/2-a1/4, -b1/2-a1, -u))
        arcptsl.push(Geom::Point3d.new(a1/2, -b1/2-a1+a1/4, 0))
        arcptsh.push(Geom::Point3d.new(a1/2, -b1/2-a1+a1/4, -u))        
 
    for i in 0..12 do
    face[i] = container.add_face arcptsl[i], arcptsh[i], arcptsh[i+1], arcptsl[i+1]
    end
    
    face[13] = container.add_face arcptsl[13], arcptsh[13], arcptsh[0], arcptsl[0]

    face0edges = []
    face0edges = face[0].edges
    face0edges[0].soft = true
    face0edges[0].smooth = true
    face0edges[2].soft = true
    face0edges[2].smooth = true
    
    face1edges = []
    face1edges = face[1].edges
    face1edges[0].soft = true
    face1edges[0].smooth = true
    face1edges[2].soft = true
    face1edges[2].smooth = true
    
    face2edges = []
    face2edges = face[2].edges
    face2edges[0].soft = true
    face2edges[0].smooth = true
    face2edges[2].soft = true
    face2edges[2].smooth = true
    
    face3edges = []
    face3edges = face[3].edges
    face3edges[0].soft = true
    face3edges[0].smooth = true
    face3edges[2].soft = true
    face3edges[2].smooth = true
    
    face4edges = []
    face4edges = face[4].edges
    face4edges[0].soft = true
    face4edges[0].smooth = true
    face4edges[2].soft = true
    face4edges[2].smooth = true    
    
    face5edges = []
    face5edges = face[5].edges
    face5edges[0].soft = true
    face5edges[0].smooth = true
    face5edges[2].soft = true
    face5edges[2].smooth = true 
    
    face6edges = []
    face6edges = face[6].edges
    face6edges[0].soft = true
    face6edges[0].smooth = true
    face6edges[2].soft = true
    face6edges[2].smooth = true     
    
    face7edges = []
    face7edges = face[7].edges
    face7edges[0].soft = true
    face7edges[0].smooth = true
    face7edges[2].soft = true
    face7edges[2].smooth = true   
    
    face8edges = []
    face8edges = face[8].edges
    face8edges[0].soft = true
    face8edges[0].smooth = true
    face8edges[2].soft = true
    face8edges[2].smooth = true      
    

    for i in 0..13 do
    face[i].material = [170, 170, 170] 
    face[i].back_material = [170, 170, 170]
    end     
         
     basel = container.add_face arcptsl
     baseh = container.add_face arcptsh
     basel.material = [170, 170, 170] 
     basel.back_material = [170, 170, 170]
     baseh.material = [170, 170, 170] 
     baseh.back_material = [170, 170, 170]

    point01 = Geom::Point3d.new (0, -b1/2-a1/2, 0)
    container.add_cpoint point01
    point02 = Geom::Point3d.new (0, -b1/2-a1/2, -u)
    container.add_cpoint point02

end

def default_parameters
    defaults = {"a", 100.mm, "b", 100.mm, "t", 3.mm, "u", 5.mm}
    defaults
end


def validate_parameters(data)
    ok = true
    if( 4*data["t"] >= data["a"] ) or ( 4*data["t"] >= data["b"] )
        UI.messagebox "Thickness must be smaller"
        ok = false
    end
    ok
end


def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "t"
            prompt = "Wall"
        when "u"
            prompt = "Height"                       
    end
    prompt
end

end

#=============================================================================

class Two_Sides_Flange < Parametric

def create_entities(data, container)

    a = data["a"].to_l 
    b = data["b"].to_l
    u = data["u"].to_l
    
    arcptsl = []
    arcptsh = []
    face = []

        arcptsl.push(Geom::Point3d.new(-a/2, -b/2-a+a/4, 0))
        arcptsh.push(Geom::Point3d.new(-a/2, -b/2-a+a/4, -u))
        arcptsl.push(Geom::Point3d.new(-a/2+a/4, -b/2-a, 0))
        arcptsh.push(Geom::Point3d.new(-a/2+a/4, -b/2-a, -u))
        arcptsl.push(Geom::Point3d.new(a/2-a/4, -b/2-a, 0))
        arcptsh.push(Geom::Point3d.new(a/2-a/4, -b/2-a, -u))
        arcptsl.push(Geom::Point3d.new(a/2, -b/2-a+a/4, 0))
        arcptsh.push(Geom::Point3d.new(a/2, -b/2-a+a/4, -u))        

        arcptsh.push(Geom::Point3d.new(a/2, b/2+a-a/4, -u))
        arcptsl.push(Geom::Point3d.new(a/2, b/2+a-a/4, 0))
        arcptsh.push(Geom::Point3d.new(a/2-a/4, b/2+a, -u))
        arcptsl.push(Geom::Point3d.new(a/2-a/4, b/2+a, 0))
        arcptsh.push(Geom::Point3d.new(-a/2+a/4, b/2+a, -u))        
        arcptsl.push(Geom::Point3d.new(-a/2+a/4, b/2+a, 0))        
        arcptsh.push(Geom::Point3d.new(-a/2, b/2+a-a/4, -u))             
        arcptsl.push(Geom::Point3d.new(-a/2, b/2+a-a/4, 0))
                 
    for i in 0..6 do
    face[i] = container.add_face arcptsl[i], arcptsh[i], arcptsh[i+1], arcptsl[i+1]
    end
    
    face[7] = container.add_face arcptsl[7], arcptsh[7], arcptsh[0], arcptsl[0]

    

    for i in 0..7 do
    face[i].material = [170, 170, 170] 
    face[i].back_material = [170, 170, 170]
    end     
         
     basel = container.add_face arcptsl
     baseh = container.add_face arcptsh
     basel.material = [170, 170, 170] 
     basel.back_material = [170, 170, 170]
     baseh.material = [170, 170, 170] 
     baseh.back_material = [170, 170, 170]

    point01 = Geom::Point3d.new (0, -b/2-a/2, 0)
    container.add_cpoint point01
    point02 = Geom::Point3d.new (0, -b/2-a/2, -u)
    container.add_cpoint point02
    point01 = Geom::Point3d.new (0, b/2+a/2, 0)
    container.add_cpoint point01
    point02 = Geom::Point3d.new (0, b/2+a/2, -u)
    container.add_cpoint point02

end

def default_parameters
    defaults = {"a", 100.mm, "b", 100.mm, "u", 5.mm}
    defaults
end

def translate_key(key)
    prompt = key
    case( key )
        when "a"
            prompt = "Width"
        when "b"
            prompt = "Depth"
        when "u"
            prompt = "Height"                       
    end
    prompt
end

end

#=============================================================================




# Add a menu to create shapes
if( not $shapes_menu_loaded )
    add_separator_to_menu("Draw")
    shapes_menu = UI.menu("Draw").add_submenu("Parametric Shapes")

    shapes_menu.add_item("Cone") { Cone.new }
    shapes_menu.add_item("Pyramid") { Pyramid.new } 
    shapes_menu.add_item("Prism") { Prism.new }
    shapes_menu.add_item("Tube") { Tube.new }
    shapes_menu.add_item("Tube Lid") { Cylinder.new }
    shapes_menu.add_item("Sq. Profile") { Jackel.new }
    shapes_menu.add_item("Sq. Prof. Lid") { Beam.new }
    shapes_menu.add_item("One Side Flange") { One_Side_Flange.new }
    shapes_menu.add_item("Two Sides Flange") { Two_Sides_Flange.new }
    shapes_menu.add_item("L Flange for 45") { L_flange45.new }
    shapes_menu.add_item("L Flange for 50") { L_flange50.new }
    shapes_menu.add_item("Sq. Prof. Diam.") { Jackel_diam.new }
    shapes_menu.add_item("Rib") { Rib.new }       
    shapes_menu.add_item("Plate") { Box.new }
    shapes_menu.add_item("Chamfered Plate") { Cham_box.new }    
    shapes_menu.add_item("U Profile") { U_profile.new }
    shapes_menu.add_item("Z Profile") { Z_profile.new }
    shapes_menu.add_item("L Profile 90") { L_profile90.new }
    shapes_menu.add_item("L Profile 45") { L_profile45.new }
       
    $shapes_menu_loaded = true
end
